** FILE:
**    SWALIBAO.FOR - part of program SWAP
** FILE IDENTIFICATION:
**    $Id: SWALIB.FOR 1.5 2000/02/11 22:59:47 Kroes Exp $
** DESCRIPTION:
**    This file contains program units of SWAP. The program units 
**    in this file are of the 3rd level and are sorted alphabetically
**    with the first letter of the program unit ranging from A - O.
**    The following program units are in this file:
**        ASSIM, ASTRO, DEVRIES, DMCNODE, FLUXES, INTEGRAL, 
**        NOCROP, OUTAFO, OUTAIR, OUTATE, OUTAUN, OUTBAL, 
**        OUTSBA, OUTTEP, OUTVAP, OUTWBA
************************************************************************
C ----------------------------------------------------------------------
      SUBROUTINE ASSIM (AMAX,EFF,LAI,KDIF,SINB,PARDIR,PARDIF,FGROS)
C ----------------------------------------------------------------------
C     Author: Daniel van Kraalingen, 1986
C     Calculates the gross CO2 assimilation rate of the whole crop, 
C     FGROS, by performing a Gaussian integration over depth in the 
C     crop canopy. At three different depths in the canopy, i.e. for
C     different values of LAI, the assimilation rate is computed for
C     given fluxes of photosynthetically active radiation, whereafter
C     integration over depth takes place. For more information: see 
C     Spitters et al. (1988). The input variables SINB, PARDIR and 
C     PARDIF are calculated in RADIAT.
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER I

      REAL*8  LAI,LAIC,KDIF,KDIRBL,KDIRT
      REAL*8  AMAX,EFF,FGL,FGROS,FGRSH,FGRSUN,FSLLA,GAUSR,PARDIF,PARDIR
      REAL*8  REFH,REFS,SCV,SINB,VISD,VISDF,VISPP,VISSHD,VIST

      DATA    GAUSR /0.3872983d0/
C ----------------------------------------------------------------------
C --- extinction coefficients KDIF,KDIRBL,KDIRT
      SCV    = 0.2d0
      REFH   = (1.0d0-SQRT(1.0d0-SCV))/(1.0d0+SQRT(1.0d0-SCV))
      REFS   = REFH*2.0/(1.0d0+1.6*SINB)
      KDIRBL = (0.5/SINB)*KDIF/(0.8*SQRT(1.0d0-SCV))
      KDIRT  = KDIRBL*SQRT(1.0d0-SCV)

C --- three point gaussian integration over LAI
      FGROS  = 0.
      DO 10 I = 1,3
        LAIC   = 0.5*LAI+GAUSR*(I-2)*LAI
C --- absorbed diffuse radiation (VIDF),light from direct
C --- origine (VIST) and direct light(VISD)
        VISDF  = (1.0d0-REFS)*PARDIF*KDIF  *EXP(-KDIF  *LAIC)
        VIST   = (1.0d0-REFS)*PARDIR*KDIRT *EXP(-KDIRT *LAIC)
        VISD   = (1.0d0-SCV) *PARDIR*KDIRBL*EXP(-KDIRBL*LAIC)
C --- absorbed flux in W/m2 for shaded leaves and assimilation
        VISSHD = VISDF+VIST-VISD
        FGRSH  = AMAX*(1.0d0-EXP(-VISSHD*EFF/AMAX))
C --- direct light absorbed by leaves perpendicular on direct
C --- beam and assimilation of sunlit leaf area
        VISPP  = (1.0d0-SCV)*PARDIR/SINB
        IF (VISPP.LE.0.0d0) FGRSUN = FGRSH
        IF (VISPP.GT.0.0d0) FGRSUN = AMAX*(1.0d0-
     $    (AMAX-FGRSH)*(1.0d0-EXP(-VISPP*EFF/AMAX))/ (EFF*VISPP))
C --- fraction of sunlit leaf area (FSLLA) and local
C --- assimilation rate (FGL)
        FSLLA  = EXP(-KDIRBL*LAIC)
        FGL    = FSLLA*FGRSUN+(1.0d0-FSLLA)*FGRSH
C --- integration
        IF (I.EQ.2) FGL = FGL*1.6
        FGROS  = FGROS+FGL
10    CONTINUE
      FGROS  = FGROS*LAI/3.6

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE ASTRO (IDAY,LAT,DAYL,DAYLP,SINLD,COSLD)
C ----------------------------------------------------------------------
C     Author: Daniel van Kraalingen, 1986
C     Computation of astronomical daylength (DAYL), photoperiodic
C     daylength (DAYLP) and two intermediate variables, SINLD and COSLD, 
C     from day number and geographic latitude. Photoperiodic daylength
C     for the effects on development rate is defined as the period that
C     the angle of the sun above the horizon exceeds -4 degrees.
C ----------------------------------------------------------------------
      IMPLICIT NONE
 
      INTEGER IDAY

      REAL*8  LAT
      REAL*8  ANGLE,AOB,COSLD,DAYL,DAYLP,DEC,PI,RAD,SINLD

      DATA    PI /3.1415926d0/,ANGLE /-4.0d0/
C ----------------------------------------------------------------------
C --- declination of the sun as a function of IDAY
      RAD   = PI/180.
      DEC   = -ASIN(SIN(23.45*RAD)*COS(2.*PI*(IDAY+10.0)/365.))
C --- some intermediate variables
      SINLD = SIN(RAD*LAT)*SIN(DEC)
      COSLD = COS(RAD*LAT)*COS(DEC)
      AOB   = SINLD/COSLD      
C --- calculation of daylenght and photoperiodic daylength
      DAYL  = 12.0*(1.0d0+2.*ASIN(AOB)/PI)
      DAYLP = 12.0*(1.0d0+2.*ASIN((-SIN(ANGLE*RAD)+SINLD)/COSLD)/PI)

      RETURN
      END

************************************************************************
      Subroutine Devries (NumNod,theta,THETAS,HeaCap,HeaCon,FQUARTZ,
     &           FCLAY,FORG)
************************************************************************
** Purpose:    Calculate soil heat capacity and conductivity for each  *
**             compartment by full de Vries model                      *     
** References:                                                         *
** Description:                                                        *
** de Vries model for soil heat capcity and thermal conductivity.      * 
** Heat capacity is calculated as average of heat capacities for each  *
** soil component. Thermal conductivity is calculated as weighted      *
** average of conductivities for each component. If theta > 0.05 liquid*
** water is assumed to be the main transport medium in calculating the *
** weights. If theta < 0.02 air is assumed to be the main transport    *
** medium (there is also an empirical adjustment to the conductivity). *
** For 0.02 < theta < 0.05 conductivity is interpolated.               *
** See: Heat and water transfer at the bare soil surface, H.F.M Ten    *
** Berge (pp 48-54 and Appendix 2)                                     *
************************************************************************
** Input:                                                              *
** NumNod - number of compartments (-)                                 *
** theta/THETAS - volumetric soil moisture/ saturated vol. s. moist (-)*
** Fquartz, Fclay and Forg - volume fractions of sand, clay and org.ma.*
** Output:                                                             *
** HeaCap - heat capacity (J/m3/K)                                     *
** HeaCon - thermal conductivity (W/m/K)                               *
************************************************************************
      Implicit None
      Include 'Param.fi'
  
*     (i) Global declarations                                          
*     (i.i) Input                                                      
      Integer NumNod
      Real*8 theta(1:MACP),THETAS(1:MACP)
*     (i.ii)
      Real*8 HeaCap(MACP), HeaCon(MACP)
*     (ii) Local declarations                                          
      Integer Node
      Real*8 kqa,kca,kwa,koa,kaa,kqw,kcw,kww,kow,kaw
      Parameter (kaa = 1.0d0, kww = 1.0d0)
      Real*8 fAir(MACP),fClay(MACP),fQuartz(MACP),fOrg(MACP)
      Real*8 HeaConDry,HeaConWet
*     (iii) Parameter declarations                                     
*     (iii.i) Physical constants                                       
*     Specific heats (J/kg/K)                                          
      REAL*8  cQuartz,cClay,cWat,cAir,cOrg
      Parameter (cQuartz = 800.0d0, cClay = 900.0d0, cWat = 4180.0d0,
     &           cAir = 1010.0d0, cOrg = 1920.0d0)
*     Density (kg/m)
      Real*8 dQuartz, dClay, dWat, dAir, dOrg
      Parameter (dQuartz = 2660.0d0, dClay = 2650.0d0, dWat = 1000.0d0,
     &           dAir = 1.2d0, dOrg = 1300.0d0)
*     Thermal conductivities (W/m/K)                                   
      REAL*8  kQuartz,kClay,kWat,kAir,kOrg
      Parameter (kQuartz = 8.8d0, kClay = 2.92d0, kWat = 0.57d0,
     &           kAir = 0.025d0, kOrg = 0.25d0)
*     
      REAL*8  GQuartz,GClay,GWat,GAir,GOrg
      Parameter (GQuartz = 0.14d0, GClay = 0.0d0 , GWat = 0.14d0, 
     &           GAir = 0.2d0, GOrg = 0.5d0)
*     (iii.ii) theta 0.02, 0.05 and 1.00                               
      REAL*8thetaDry,thetaWet
      Parameter (thetaDry = 0.02d0, thetaWet = 0.05d0)    
C ----------------------------------------------------------------------
*     (0) Weights for each component in conductivity calculations      
*     (Calculate these but define as parameters in later version)     
      kaw = 0.66 / (1.0d0 + ((kAir/kWat) - 1.0d0) * GAir) + 0.33d0 /
     &      (1.0d0 + ((kAir/kWat) - 1.0d0) * (1.0d0 - 2.0 * GAir)) 
      kqw = 0.66 / (1.0d0 + ((kQuartz/kWat) - 1.0d0) * GQuartz)+0.33d0 /
     &      (1.0d0 + ((kQuartz/kWat) - 1.0d0) * (1.0d0 - 2.0 * GQuartz)) 
      kcw = 0.66 / (1.0d0 + ((kClay/kWat) - 1.0d0) * GClay) + 0.33d0 /
     &      (1.0d0 + ((kClay/kWat) - 1.0d0) * (1.0d0 - 2.0 * GClay)) 
      kow = 0.66 / (1.0d0 + ((kOrg/kWat) - 1.0d0) * GOrg) + 0.33d0 /
     &      (1.0d0 + ((kOrg/kWat) - 1.0d0) * (1.0d0 - 2.0 * GOrg)) 
      kwa = 0.66 / (1.0d0 + ((kWat/kAir) - 1.0d0) * GWat) + 0.33d0 /
     &      (1.0d0 + ((kWat/kAir) - 1.0d0) * (1.0d0 - 2.0 * GWat)) 
      kqa = 0.66 / (1.0d0 + ((kQuartz/kAir) - 1.0d0) * GQuartz)+0.33d0 /
     &      (1.0d0 + ((kQuartz/kAir) - 1.0d0) * (1.0d0 - 2.0 * GQuartz)) 
      kca = 0.66 / (1.0d0 + ((kClay/kAir) - 1.0d0) * GClay) + 0.33d0 /
     &      (1.0d0 + ((kClay/kAir) - 1.0d0) * (1.0d0 - 2.0 * GClay)) 
      koa = 0.66 / (1.0d0 + ((kOrg/kAir) - 1.0d0) * GOrg) + 0.33d0 /
     &      (1.0d0 + ((kOrg/kAir) - 1.0d0) * (1.0d0 - 2.0 * GOrg)) 

      Do Node = 1,NumNod

*        (1) Air fraction
         fAir(Node) = THETAS(Node) - theta(Node)

*        (2) Heat capacity (W/m3/K) is average of heat capacities for  
*        all components (multiplied by density for correct units)      
         HeaCap(Node) = fQuartz(Node)*dQuartz*cQuartz + fClay(Node)*
     &                  dClay*cClay + theta(Node)*dWat*cWat + 
     &                  fAir(Node)*dAir*cAir +
     &                  fOrg(Node)*dOrg*cOrg

*        (3) Thermal conductivity (W/m/K) is weighted average of       
*        conductivities of all components                              
*        (3.1) Dry conditions (include empirical correction) (eq. 3.44)
         If (theta(Node).LE.thetaDry) Then
            HeaCon(Node) = 1.25 * (kqa*fQuartz(Node)*kQuartz + 
     &                      kca*fClay(Node)*kClay +
     &                      kaa*fAir(Node)*kAir +
     &                      koa*fOrg(Node)*kOrg +
     &                      kwa*theta(Node)*kWat) /
     &   (kqa * fQuartz(Node) + kca * fClay(Node) + kaa * fAir(Node) +
     &    koa * fOrg(Node) + kwa * theta(Node))

*        (3.2) Wet conditions  (eq. 3.43)                              
         Else If (theta(Node).GE.thetaWet) Then
            HeaCon(Node) = (kqw*fQuartz(Node)*kQuartz + 
     &                      kcw*fClay(Node)*kClay +
     &                      kaw*fAir(Node)*kAir +
     &                      kow*fOrg(Node)*kOrg +
     &                      kww*theta(Node)*kWat) /
     &   (kqw * fQuartz(Node) + kcw * fClay(Node) + kaw * fAir(Node) +
     &    kow * fOrg(Node) + kww * theta(Node))

*        (3.3) dry < theta < wet (interpolate)
         Else
*           (3.3.1) Conductivity for theta = 0.02                      
            HeaConDry = 1.25 * (kqa*fQuartz(Node)*kQuartz + 
     &                      kca*fClay(Node)*kClay +
     &                      kaa*fAir(Node)*kAir +
     &                      koa*fOrg(Node)*kOrg +
     &                      kwa*thetaDry*kWat) /
     &   (kqa * fQuartz(Node) + kca * fClay(Node) + kaa * fAir(Node) +
     &    koa * fOrg(Node) + kwa * thetaDry)
*           (3.3.1) Conductivity for theta = 0.05                      
            HeaConWet = (kqw*fQuartz(Node)*kQuartz + 
     &                      kcw*fClay(Node)*kClay +
     &                      kaw*fAir(Node)*kAir +
     &                      kow*fOrg(Node)*kOrg +
     &                      kww*thetaWet*kWat) /
     &   (kqw * fQuartz(Node) + kcw * fClay(Node) + kaw * fAir(Node) +
     &    kow * fOrg(Node) + kww * thetaWet)
*         (3.3.3) Interpolate                                          
          HeaCon(Node) = HeaConDry + (theta(Node)-thetaDry) * 
     &                   (HeaConWet - HeaConDry) 
     &                 / (thetaWet - thetaDry)      
         End If

C ---    conversion of capacity from J/m3/K to J/cm3/K
         HEACAP(NODE) = HEACAP(NODE)*1.0d-6

C ---    conversion of conductivity from W/m/K to J/cm/K/d
         HEACON(NODE) = HEACON(NODE)*864.0

      End Do

      Return
      End 

C ----------------------------------------------------------------------
      SUBROUTINE INTEGRAL (PTRA,DT,PEVA,REVA,QBOT,POND,QTOP,NRAI,NIRD,
     & PONDMX,NUMNOD,QROT,QDRTOT,QDRAIN,DQROT,DPTRA,IQROT,INQROT,IQDRA,
     & NRLEVS,INQDRA,QDRA,GRAI,GIRD,IINTC,IPTRA,IPEVA,IEVAP,IRUNO,
     & IPREC,IQBOT,CQROT,CQDRA,CQDRAIN,CPTRA,CPEVA,CEVAP,CRUNO,
     & CGRAI,CNRAI,CGIRD,CNIRD,CQBOT,CQTOP,PONDM1,WTOPLAT,RAPDRA,
     & IQDRAR,IQCRACK,QCRACK,CQDRAR,CQCRACK,ACRACK,ARFLUX,T,
     & WLS,RSRO,RUNOTS,QROSUM,SWCRACK,SWDRA,IGRAI,IGIRD,INRAI,INIRD,
     & QCOMP,QDARCY,CQDARCY,Volini,Volact,CrackW)
C ----------------------------------------------------------------------
C     Date               : 4/2/2000                                        
C     Purpose            : calculation of intermediate values and     
C                          cumulative values          
C     Subroutines called : -                                           
C     Functions called   : -                     
C     File usage         : -                                           
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      INTEGER NUMNOD,NRLEVS,SWCRACK,SWDRA

      REAL*8  PTRA,DT,PEVA,REVA,POND,QTOP,NRAI,NIRD,PONDMX,QROT(MACP)
      REAL*8  QDRTOT,QDRAIN(5),DQROT,DPTRA,IQROT,INQROT(MACP),IQDRA
      REAL*8  INQDRA(5,MACP),IGRAI,IGIRD
      REAL*8  QDRA(5,MACP),IINTC,IPTRA,IPEVA,IEVAP,IRUNO,IPREC
      REAL*8  IQBOT,GRAI,GIRD,CQROT,CQDRA,CQDRAIN(5),CPTRA,CPEVA,CEVAP
      REAL*8  CRUNO,QROSUM,QCOMP,T,QDARCY,CQDARCY
      REAL*8  CGRAI,CNRAI,CGIRD,CNIRD,CQBOT,CQTOP,QBOT,ACRACK
      REAL*8  PONDM1,WTOPLAT,RAPDRA,IQDRAR,IQCRACK,QCRACK,CQDRAR,CQCRACK
      REAL*8  WLS,RSRO,RUNOTS,ARFLUX ,INRAI,INIRD,Volini,Volact,Crackw        
      
C ----------------------------------------------------------------------
C --- local
      INTEGER node,level

      REAL*8  qrotts,qdrats,ptrats,pevats,revats,masdif
      REAL*8  qbotts,difpond,wbalance
C ----------------------------------------------------------------------
              
C --- potential transpiration of this timestep
      ptrats = ptra * dt

C --- potential soil evaporation of this timestep
      pevats = peva * dt

C --- reduced soil evaporation of this timestep
      revats = reva * dt

C --- flux lower boundary of this timestep
      qbotts = qbot*dt

C --- runoff of this timestep, take physical limits into account
      pondm1 = pond
      difpond = (qtop + (ARFLUX + NIRD) * (1.0d0 - acrack)) * dt 
     &        - revats - wtoplat
      difpond = min(difpond, 100.0*dt)
      pond = pond + difpond
      if (pond.lt.1.0d-6) pond=0.0

C --- height ponding layer determines amount runoff or runon
      runots = 0.0
      if (SWDRA.LT.2) then
        if (pond.gt.pondmx .and. .not. (SWCRACK.EQ.1)) then
            runots = pond - pondmx
        endif
      else
         if (pond.gt.pondmx .or. wls.gt.pondmx) then
           if (pond.gt.pondmx) then
             if (wls.gt.pondmx) then
                runots = pond - wls
             else
                runots = pond - pondmx
             endif
           else
             runots = pondmx - wls
           endif
C ---      reduce runots so that the rate corresponds to a drainage flux
C ---      with drainage resistance of RSRO
           runots = runots*dt/rsro
         endif
      endif
      pond = pond - runots


C --- total root extraction of this timestep
      qrotts = QROSUM * dt

C --- total drainage flux of this timestep
      qdrats = qdrtot * dt

C --- add time step fluxes to daily totals
      dqrot = dqrot + qrotts
      dptra = dptra + ptrats

C --- add time step fluxes to intermediate totals
      iqrot  = iqrot + qrotts
      do 300 node = 1,numnod
300   inqrot(node) = inqrot(node) + qrot(node) * dt
      iqdra = iqdra + qdrats + rapdra*dt
      do 410 node = 1,numnod
        do 400 level = 1,nrlevs
          inqdra(level,node) = inqdra(level,node)+qdra(level,node)*dt
400     continue
410   continue

      iintc   = iintc + ((GRAI-NRAI)+(GIRD-NIRD))*dt
      iptra   = iptra + ptrats
      ipeva   = ipeva + pevats
      ievap   = ievap + revats
      iruno   = iruno + runots
      iprec   = iprec + (GRAI+GIRD)*dt
      igrai   = igrai + grai*dt
      igird   = igird + gird*dt
      inrai   = inrai + nrai*dt
      inird   = inird + nird*dt
      iqbot   = iqbot + qbotts
      iqdrar  = iqdrar + rapdra*dt 
      iqcrack = iqcrack +qcrack*dt

C --- add time step fluxes to total cumulative values
      cqrot   = cqrot + qrotts
      cqdra   = cqdra + qdrats
      cptra   = cptra + ptrats
      cpeva   = cpeva + pevats
      cevap   = cevap + revats
      cruno   = cruno + runots
      CGRAI   = CGRAI + GRAI*DT
      CNRAI   = CNRAI + NRAI*DT
      CGIRD   = CGIRD + GIRD*DT
      CNIRD   = CNIRD + NIRD*DT
      cqbot   = cqbot + qbotts
      cqdrar  = cqdrar + rapdra*dt
      cqcrack = cqcrack + qcrack*dt 
      cqtop   = cqtop + qtop*dt
      cqdarcy = cqdarcy + qdarcy*dt
      do 150 level = 1,5
        cqdrain(level) = cqdrain(level) + qdrain(level)*dt
150   continue

c --- compensate water balance error of this time step during remaining day part
c --- cumulative water balance error
      wbalance = cnrai + cnird - cruno - cqrot - cevap - cqdra - cqdrar
     &         + cqbot + volini - volact - pond - crackw

c --- in case of ponding, correct ponding height with water balance error
      if (pond .gt. 1.0d-8) then
        if ((wbalance+pond).gt.pondmx .and. swdra.lt.2) then
           wbalance = wbalance - (pondmx - pond)
           pond = pondmx
        else
          if (wbalance + pond .lt. 0.0d0) then
            wbalance = wbalance + pond
            pond = 0.0
          else
            pond = pond + wbalance
            wbalance = 0.0
          endif
        endif
      endif

c --- adjust remaing water balance error in qtop with qcomp(ensation)
c --- during remaining time of this day
      qcomp =  - wbalance / (aint(t+1.0d0) - t)

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE NOCROP (DRZ,LAI,CF,FLCROP)
C ----------------------------------------------------------------------
      REAL*8  DRZ,LAI,CF
      LOGICAL FLCROP
C ----------------------------------------------------------------------
      DRZ    = 0.0
      LAI    = 0.0
      CF     = 0.0
      FLCROP = .FALSE.

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE OUTAFO (AFO,AFONAM,HAFOD,STASTR,ENVSTR,
     &  NUMNOD,OUTPER,PERIOD,TCUM,POND,GWL,THETA,H,INQ,
     &  INQROT,NRLEVS,INQDRA,IEVAP,IPEVA,IPTRA,IRUNO,
     &  NUMLAY,BOTCOM,DZ,THETAS,BRUNY,ERUNY,BRUND,ERUND,
     &  LAYER,FLGENU,ALFAMG,THETAR,COFGEN,HTABLE,THETLO,
     &  SWAFO,TEMPI,IGRAI,IGIRD,INRAI,INIRD,GC,LAI,TAV,TSoil)
C ----------------------------------------------------------------------
C     Date               : 29/9/99           
C     Purpose            : ANIMO output: formatted hydrological data
C     Subroutines called : -              
C     Functions called   : THENODE 
C     File usage         : AFONAM
C ---------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

      INTEGER AFO,OUTPER,PERIOD,LAY,LEVEL,NODE
      INTEGER NUMLAY,NUMNOD,NRLEVS,BOTCOM(MAHO),LAYER(MACP)
      INTEGER BRUNY,ERUNY,BRUND,ERUND,THETLO(MAHO),SWAFO

      REAL*8  GWL,INQDRA(5,MACP),IPREC,IINTC,IEVAP,POND,TCUM
      REAL*8  IPEVA,IPTRA,IRUNO,INQ(MACP+1),INQROT(MACP),DZ(MACP)
      REAL*8  TEMPI(MACP),IGRAI,IGIRD,INRAI,INIRD,GC,LAI,TAV 
      REAL*8  THETA(MACP),H(MACP),THETAS(MACP),THENODE
      REAL*8  ALFAMG(MACP),THETAR(MACP),COFGEN(8,MAHO),HTABLE(MAHO,99)
      REAL*8  DVAL(MAHO),TSoil(MACP)

      CHARACTER AFONAM*8,STASTR*7,ENVSTR*50,FILNAM*12

      LOGICAL HAFOD,FLGENU(MAHO)
C ----------------------------------------------------------------------
      IF (.NOT.HAFOD) THEN
C ---   open output file
        CALL SHIFTR (AFONAM)
        FILNAM = AFONAM//'.AFO'
        If (SWAFO.EQ.1) then
          FILNAM = AFONAM//'.AFO'
        elseif (SWAFO.eq.2) then
          FILNAM = AFONAM//'.PFO'
        endif
        CALL SHIFTL (FILNAM)
        CALL SHIFTR (ENVSTR)
        CALL Findunit (72,AFO)
        OPEN (AFO,FILE=ENVSTR//FILNAM,STATUS=STASTR,
     &  FORM ='FORMATTED',ACCESS='SEQUENTIAL')

        if (period.eq.0) period = -10
        write (AFO,4000) BRUNY,ERUNY, 
     &  float(BRUND-1),float(ERUND),float(PERIOD)
        if (period.eq.-10) period = 0

        write (AFO,4010) numnod,numlay,nrlevs
        write (AFO,4010) (botcom(lay),lay=1,numlay)
        write (AFO,4020) (real(thetaS(botcom(lay))), lay=1,numlay)

        DO 4 lay = 1,numlay
          DVAL(lay) = thenode(botcom(lay),-100.d0,LAYER,
     &                FLGENU,COFGEN,HTABLE,THETLO,ALFAMG,THETAR,THETAS)
4       CONTINUE
        write (AFO,4020) (REAL(DVAL(lay)),lay=1,numlay) 

        DO 6 lay = 1,numlay
          DVAL(lay) = thenode(botcom(lay),-15849.0d0,LAYER,
     &                FLGENU,COFGEN,HTABLE,THETLO,ALFAMG,THETAR,THETAS)
6       CONTINUE
        write (AFO,4020) (REAL(DVAL(lay)),lay=1,numlay) 

        write (AFO,4020) (0.01*dz(node),node=1,numnod)
        write (AFO,4020) (real(theta(node)),node=1,numnod)
        write (AFO,4020) -0.01*gwl,0.01*pond
        If (SWAFO.EQ.2) then
          write (AFO,4020) (tempi(node),node = 1,numnod) 
        endif
4000    format (2(1X,I8),3(1X,F8.0),1X,i8) 
4010    format (5(1X,i8))
4020    format (8(1X,F8.4))

        HAFOD = .TRUE.
        RETURN
      ENDIF

C --- formatted output of data - dynamic part
      if (HAFOD) then
        if (SWAFO.eq.1) then
          write (AFO,30) TCUM,
     &    0.01*(iGrai+IGird)/OUTPER,
     &    0.01*((iGrai-iNrai)+(iGird-iNird))/OUTPER,
     $    0.01*ievap/OUTPER,
     &    0.0,
     $    0.01*ipeva/OUTPER,
     $    0.01*iptra/OUTPER,
     &    0.01*iruno/OUTPER,
     $    -0.01*gwl,0.01*pond
        elseif (SWAFO.EQ.2) then
          write (AFO,32) TCUM,
     &    0.01*iGrai/OUTPER,
     $    0.01*iGird/OUTPER,
     &    0.01*(iGrai-iNrai)/OUTPER,
     $    0.01*(iGird-iNird)/OUTPER,
     $    0.01*ievap/OUTPER,
     &    0.0,
     $    0.01*ipeva/OUTPER,
     $    0.01*iptra/OUTPER,
     &    0.01*iruno/OUTPER,
     $    -0.01*gwl,0.01*pond
        endif
        write (AFO,40) (h(node)                 ,node=1,numnod)
        write (AFO,50) (theta(node)             ,node=1,numnod)
        write (AFO,50) (0.01*inqrot(node)/OUTPER,node=1,numnod)
        write (AFO,50) (-0.01*inq(node)/OUTPER  ,node=1,numnod+1)
        if (nrlevs.gt.0) then
          do 300 level=1,nrlevs
            write (AFO,50) (0.01*inqdra(level,node)/OUTPER,
     $        node=1,numnod)
 300      continue
        end if
        If (SWAFO.EQ.2) then
          GC = Min(LAI/3.0,1.0d0)
          write (AFO,'(2(1x,f7.4))') GC,LAI
          write (AFO,'(1X,F7.2)') TAV
          write (AFO,4020) (TSoil(node),node = 1,numnod) 
        endif
      end if

30    format (1x,f6.0,7(1x,f7.6),2(1x,f7.4))
32    format (1x,f6.0,9(1x,f7.6),2(1x,f7.4))
40    format (8(1x,e8.3e1))
50    format (10(1x,f7.5))

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE OUTAIR (AIR,AIRNAM,HAIRD,STASTR,ENVSTR,
     &  YEAR,DAYNR,TCUM,GIRD,CIRR)
C ----------------------------------------------------------------------
C     Date               : 24/10/1997           
C     Purpose            : ANIMO/Pestla output: irrigations           
C     Subroutines called : -              
C     Functions called   : -       
C     File usage         : AIRNAM
C ---------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER AIR,YEAR,DAYNR
c     INTEGER(2) iday,imon,iyr

      REAL*8  TCUM,GIRD,CIRR

      CHARACTER AIRNAM*8,STASTR*7,IRSTRING*12,ENVSTR*50,FILNAM*12

      LOGICAL HAIRD
C ----------------------------------------------------------------------
      IF (.NOT.HAIRD) THEN
C ---   open output file
        CALL SHIFTR (AIRNAM)
        FILNAM = AIRNAM//'.AIR'
        CALL SHIFTL (FILNAM)
        CALL SHIFTR (ENVSTR)
        CALL Findunit (74,AIR)
        OPEN (AIR,FILE=ENVSTR//FILNAM,STATUS=STASTR,
     &  FORM ='FORMATTED',ACCESS='SEQUENTIAL')
C ---   write header
        WRITE (AIR,'(2A)')'* Filename     : ',ENVSTR//FILNAM  
        WRITE (AIR,'(A)') '* FileContent  : irrigations'
        WRITE (AIR,'(A)') '* ProducerId   :     '
c       CALL GETDAT (iyr,imon,iday)
c       WRITE (AIR,'(A,2I3,I5)') '* ProducerDate :',iday,imon,iyr
        WRITE (AIR,'(A)') '* ProducerDate :'
        HAIRD = .TRUE.
      ENDIF

      IRSTRING = '''IRRIGATION'''
      WRITE (AIR,'(A,I5,1X,I3,1X,I5,2F10.3)') IRSTRING,YEAR,
     & DAYNR,INT(TCUM),GIRD/100,CIRR

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE OUTATE (ATE,ATENAM,HATED,STASTR,ENVSTR,NUMNOD,
     &  YEAR,DAYNR,TCUM,TEMPI,TSoil,TAV)
C ----------------------------------------------------------------------
C     Date               : 29/9/99          
C     Purpose            : ANIMO output: soil temperatures           
C     Subroutines called : -              
C     Functions called   : -       
C     File usage         : ATENAM
C ---------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi' 

      INTEGER NUMNOD,YEAR,DAYNR,ATE,I

      REAL*8  TCUM,TAV,TEMPI(MACP)
      REAL*8  TSoil(MACP)

      CHARACTER ATENAM*8,STASTR*7,FILNAM*12
      CHARACTER ENVSTR*50

      LOGICAL HATED

      SAVE

C ----------------------------------------------------------------------
      IF (.NOT.HATED) THEN
C ---   open output file
        CALL SHIFTR (ATENAM)
        FILNAM = ATENAM//'.ATE'
        CALL SHIFTL (FILNAM)
        CALL SHIFTR (ENVSTR)
        CALL Findunit (76,ATE)
        OPEN (ATE,FILE=ENVSTR//FILNAM,STATUS=STASTR,
     &  FORM ='FORMATTED',ACCESS='SEQUENTIAL',RECL=22+numnod*6)
C ---   write header
        WRITE (ATE,'(2A)')'* Filename     : ',ENVSTR//FILNAM  
        WRITE (ATE,'(A)') '* FileContent  : soil temperature profiles'
        WRITE (ATE,'(A)') '* ProducerId   :     '
        WRITE (ATE,'(A)') '* ProducerDate :'
C ---   write initial temperature profile
        WRITE (ATE,'(I4,1X,I3,1X,I5,100F6.1)') YEAR,DAYNR,INT(TCUM),
     &         TEMPI(1),(TEMPI(I),I=1,NUMNOD)

        HATED = .TRUE.
      ENDIF

      WRITE (ATE,'(I4,1X,I3,1X,I5,100F6.1)') YEAR,DAYNR,INT(TCUM),TAV,
     &      (TSoil(I),I=1,NUMNOD)

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE OUTAUN (AUN,AUNNAM,HAUND,STASTR,ENVSTR,
     &  NUMNOD,OUTPER,PERIOD,TCUM,POND,GWL,THETA,H,INQ,
     &  INQROT,NRLEVS,INQDRA,IEVAP,IPEVA,IPTRA,IRUNO,
     &  NUMLAY,BOTCOM,DZ,THETAS,BRUNY,ERUNY,BRUND,ERUND,
     &  LAYER,FLGENU,ALFAMG,THETAR,COFGEN,HTABLE,THETLO,
     &  SWAUN,TEMPI,IGRAI,IGIRD,INRAI,INIRD,GC,LAI,TAV,TSoil)
C ----------------------------------------------------------------------
C     Date               : 29/11/99           
C     Purpose            : ANIMO output: unformatted hydrological data
C     Subroutines called : SHIFTR,SHIFTL
C     Functions called   : THENODE 
C     File usage         : AUNNAM
C ---------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.fi'

      INTEGER AUN,OUTPER,PERIOD,LAY,LEVEL,NODE
      INTEGER NUMLAY,NUMNOD,NRLEVS,BOTCOM(MAHO),LAYER(MACP)
      INTEGER BRUNY,ERUNY,BRUND,ERUND,THETLO(MAHO),SWAUN

      REAL*8  GWL,INQDRA(5,MACP),IPREC,IINTC,IEVAP,POND,TCUM
      REAL*8  IPEVA,IPTRA,IRUNO,INQ(MACP+1),INQROT(MACP),DZ(MACP)
      REAL*8  TEMPI(MACP),IGRAI,IGIRD,INRAI,INIRD,GC,LAI,TAV 
      REAL*8  THETA(MACP),H(MACP),THETAS(MACP),THENODE
      REAL*8  ALFAMG(MACP),THETAR(MACP),COFGEN(8,MAHO),HTABLE(MAHO,99)
      REAL*8  DVAL(MAHO),TSoil(MACP) 

      CHARACTER AUNNAM*8,STASTR*7,ENVSTR*50,FILNAM*12

      LOGICAL HAUND,FLGENU(MAHO)
C ----------------------------------------------------------------------
      IF (.NOT.HAUND) THEN 
C --- open output file
        CALL SHIFTR (AUNNAM)
        If (SWAUN.EQ.1) then
          FILNAM = AUNNAM//'.AUN'
        elseif (SWAUN.eq.2) then
          FILNAM = AUNNAM//'.PUN'
        endif
        CALL SHIFTL (FILNAM)
        CALL SHIFTR (ENVSTR)
        CALL Findunit (72,AUN)
        OPEN (AUN,FILE=ENVSTR//FILNAM,STATUS=STASTR,
     &  FORM ='UNFORMATTED',ACCESS='SEQUENTIAL')

        if (period.eq.0) period = -10
        write (AUN) BRUNY,ERUNY, 
     &  float(BRUND-1),float(ERUND),float(PERIOD)
        if (period.eq.-10) period = 0
        write (AUN) numnod,numlay,nrlevs
        write (AUN) (botcom(lay),lay=1,numlay)
        write (AUN) (real(thetaS(botcom(lay))), lay=1,numlay)

        DO 4 lay = 1,numlay
          DVAL(lay) = thenode(botcom(lay),-100.d0,LAYER,
     &                FLGENU,COFGEN,HTABLE,THETLO,ALFAMG,THETAR,THETAS)
4       CONTINUE
        write (AUN) (REAL(DVAL(lay)),lay=1,numlay) 

        DO 6 lay = 1,numlay
          DVAL(lay) = thenode(botcom(lay),-15849.0d0,LAYER,
     &                FLGENU,COFGEN,HTABLE,THETLO,ALFAMG,THETAR,THETAS)
6       CONTINUE

        write (AUN) (REAL(DVAL(lay)),lay=1,numlay) 

        write (AUN) (0.01*real(dz(node)),node=1,numnod)
        write (AUN) (real(theta(node)),node=1,numnod)
        write (AUN) -0.01*real(gwl),0.01*real(pond)
        If (SWAUN.EQ.2) then
          write (AUN) (real(tempi(node)),node = 1,numnod) 
        endif
        HAUND = .TRUE.
        RETURN
      ENDIF

C --- unformatted output of data - dynamic part
      if (HAUND) then
        if (SWAUN.eq.1) then
          write (AUN) real(TCUM),
     &    0.01*real(iGrai+IGird)/OUTPER,
     &    0.01*(real(iGrai-iNrai)+real(iGird-iNird))/OUTPER,
     $    0.01*real(ievap)/OUTPER,
     &    0.0,
     $    0.01*real(ipeva)/OUTPER,
     $    0.01*real(iptra)/OUTPER,
     &    0.01*real(iruno)/OUTPER,
     $    -0.01*real(gwl),0.01*real(pond)
        elseif (SWAUN.EQ.2) then
          write (AUN) real(TCUM),
     &    0.01*real(iGrai)/OUTPER,
     $    0.01*real(iGird)/OUTPER,
     &    0.01*real(iGrai-iNrai)/OUTPER,
     $    0.01*real(iGird-iNird)/OUTPER,
     $    0.01*real(ievap)/OUTPER,
     &    0.0,
     $    0.01*real(ipeva)/OUTPER,
     $    0.01*real(iptra)/OUTPER,
     &    0.01*real(iruno)/OUTPER,
     $    -0.01*real(gwl),0.01*real(pond)
        endif
        write (AUN) (real(h(node))           ,node=1,numnod)
        write (AUN) (real(theta(node))       ,node=1,numnod)
        write (AUN) (0.01*real(inqrot(node))/OUTPER,node=1,numnod)
        write (AUN) (-0.01*real(inq(node))/OUTPER  ,node=1,numnod+1)
        if (nrlevs.gt.0) then
          do 310 level=1,nrlevs
            write (AUN) (0.01*real(inqdra(level,node))/OUTPER,
     $        node=1,numnod)
 310      continue
        end if
        If (SWAUN.EQ.2) then
          GC = Min(LAI/3.0,1.0d0)
          write (AUN) real(GC),real(LAI)
          write (AUN) real(TAV)
          write (AUN) (real(TSoil(node)),node = 1,numnod) 
        endif
      end if

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE OUTBAL (OUTNAM,ENVSTR,DAYNR,YEAR,BRUND,BRUNY,BAYRD,
     &  BAYRY,CGRAI,CNRAI,CGIRD,CNIRD,CRUNO,CQROT,CEVAP,CQBOT,
     &  CQDRAIN,CQDRA,CQDRAR,VOLACT,VOLINI,Z,DZ,SWDRA,NUMNOD,
     &  NRLEVS,SAMINI,SAMPRO,SAMCRA,SQPREC,SQIRRIG,SQBOT,DECTOT,ROTTOT,
     &  SQRAP,SQDRA,SWSOLU,POND)
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : write overview balances to OUTNAM.BAL file
C     Subroutines called : SHIFTR,SHIFTL,NRTODA
C     Functions called   : -
C     File usage         : OUTNAM.BAL
C ---------------------------------------------------------------------
      IMPLICIT NONE
      include 'Param.Fi'

      INTEGER   DAYNR,YEAR,BRUND,BRUNY,BAYRD,BAYRY,BAL,SWDRA
      INTEGER   DSTART,MSTART,YSTART,DEND,MEND,YEND,NUMNOD,NRLEVS,I
      INTEGER   SWSOLU

      REAL*8    CEVAP,CGIRD,CGRAI,CNIRD,CNRAI,CQBOT,INTERC
      REAL*8    CQROT,CRUNO,CQDRAR,CQDRA,CQDRAIN(5),DZ(MACP),Z(MACP)
      REAL*8    SAMINI,SAMPRO,SAMCRA,SQPREC,SQIRRIG,SQBOT,DECTOT,ROTTOT
      REAL*8    SQRAP,SQDRA,POND
      REAL*8    VOLACT,VOLINI

      CHARACTER OUTNAM*8,FILNAM*12,ENVSTR*50
C ----------------------------------------------------------------------
C --- compose filename
        CALL SHIFTR (OUTNAM)
        FILNAM = OUTNAM//'.BAL'
        CALL SHIFTL (FILNAM)
        CALL SHIFTR (ENVSTR)

C --- open output file
        CALL FINDUNIT (10,BAL)
        OPEN (BAL,FILE = ENVSTR//FILNAM, STATUS = 'UNKNOWN')

C --- start date of sub-run
      IF (BAYRY.GT.BRUNY.OR.
     &  ((BAYRY.EQ.BRUNY).AND.(BAYRD.GT.BRUND))) THEN
        CALL NRTODA (BAYRY,BAYRD,MSTART,DSTART)
        YSTART = BAYRY
      ELSE
        CALL NRTODA (BRUNY,BRUND,MSTART,DSTART)
        YSTART = BRUNY
      ENDIF

C --- end date of sub-run
      IF (DAYNR.GT.1) THEN
        CALL NRTODA (YEAR,(DAYNR-1),MEND,DEND)
          YEND = YEAR
      ELSE
          MEND = 12
          DEND = 31
          YEND = YEAR - 1
      ENDIF

C --- write output record

      WRITE (BAL,20) DSTART,MSTART,YSTART,DEND,MEND,YEND
      WRITE (BAL,22) (-Z(NUMNOD) + 0.5*DZ(NUMNOD))

      IF (SWSOLU .EQ. 1) THEN
          WRITE (BAL,24) (VOLACT+POND),(SAMPRO+SAMCRA),VOLINI,SAMINI,
     &    (VOLACT+POND-VOLINI),(SAMPRO+SAMCRA-SAMINI)
      ELSE
          WRITE (BAL,25) (VOLACT+POND),VOLINI,(VOLACT+POND-VOLINI)
      ENDIF

      INTERC = CGRAI+CGIRD-CNRAI-CNIRD
      WRITE (BAL,26) CGRAI,INTERC,CGIRD,CRUNO,CQBOT,CQROT,CEVAP,CQDRAR

      IF (SWDRA .NE. 0) THEN
        DO 100 I = 1,NRLEVS
          WRITE (BAL,28) I,CQDRAIN(I)
 100    CONTINUE
      ENDIF
      WRITE (BAL,30) (CGRAI+CGIRD+CQBOT),(INTERC+CRUNO+CQROT+CEVAP+
     &   CQDRAR+CQDRA)

      IF (SWSOLU .EQ. 1) THEN
        WRITE (BAL,34) SQPREC,DECTOT,SQIRRIG,ROTTOT,SQBOT,SQRAP,SQDRA
        WRITE (BAL,36) (SQPREC+SQIRRIG+SQBOT),
     &    (DECTOT+ROTTOT+SQRAP+SQDRA)
      ENDIF

      CLOSE (BAL)

 20   FORMAT('Period',t20,':',t22,i2,'/',i2.2,'/',i4,
     &   ' until  ',i2,'/',i2.2,'/',i4)
 22   FORMAT('Depth soil profile',t20,':',f8.2,' cm')
 24   FORMAT(/,T13,'Water storage',t36,'Solute storage',/,
     &   'Final',t9,':',t15,f8.2,' cm',t31,e12.4,' mg/cm2',/,
     &   'Initial',t9,':',t15,f8.2,' cm',t31,e12.4,' mg/cm2',/,
     &    t13,13('='),t33,17('='),/,
     &   'Change',t15,f8.2,' cm',t31,e12.4,' mg/cm2')
 25   FORMAT(/,T13,'Water storage',/,'Final',t9,':',t15,f8.2,' cm',/,
     &   'Initial',t9,':',t15,f8.2,' cm',/,
     &    t13,13('='),/,'Change',t15,f8.2,' cm')
 26   FORMAT(//,'Water balance components (cm)',//,
     &   'In',t30,'Out',/,25('='),t30,28('='),/,
     &   'Rain',t16,':',f9.2,t30,'Interception',t48,':',f9.2,/,
     &   'Irrigation',t16,':',f9.2,t30,'Runoff',t48,':',f9.2,/,
     &   'Bottom flux',t16,':',f9.2,t30,'Transpiration',t48,':',f9.2,
     &   /,t30,'Soil evaporation',t48,':',f9.2,/,
     &   t30,'Crack flux',t48,':',f9.2)
 28   FORMAT(t30,'Drainage level',i2,t48,':',f9.2)
 30   FORMAT(25('='),t30,28('='),/,
     &   'Sum',t16,':',f9.2,t30,'Sum',t48,':',f9.2)
 34   FORMAT(//,'Solute balance components (mg/cm2)',//,
     &   'In',t30,'Out',/,25('='),t30,28('='),/,
     &   'Rain',t13,':',e12.4,t30,'Decomposition',t45,':',e12.4,/,
     &   'Irrigation',t13,':',e12.4,t30,'Root uptake',t45,':',e12.4,/,
     &   'Bottom flux',t13,':',e12.4,t30,'Cracks',t45,':',e12.4,/,
     &   t30,'Drainage',t45,':',e12.4)
 36   FORMAT(25('='),t30,28('='),/,
     &   'Sum',t13,':',e12.4,t30,'Sum',t45,':',e12.4)

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE OUTINC (OUTNAM,INC,HINCD,ENVSTR,STASTR,DAYNR,YEAR,
     &  IGRAI,IGIRD,IINTC,IRUNO,IPTRA,IQROT,IPEVA,IEVAP,IQDRA,
     &  IQDRAR,IQBOT,T)
C ----------------------------------------------------------------------
C     Date               : 8/04/99
C     Purpose            : write water balance increments to OUTNAM.WBA file
C     Subroutines called : SHIFTR,SHIFTL,NRTODA
C     Functions called   : -
C     File usage         : OUTNAM.INC
C ---------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER   INC,DAYNR,YEAR,D,M

      REAL*8    IGRAI,IGIRD,IINTC,IRUNO,IPTRA,IQROT,IPEVA,IEVAP,IQDRA
      REAL*8    IQDRAR,IQBOT,T

      CHARACTER OUTNAM*8,FILNAM*12,ENVSTR*50,STASTR*7

      LOGICAL   HINCD
C ----------------------------------------------------------------------
      IF (.NOT.HINCD) THEN

C --- compose filename
        CALL SHIFTR (OUTNAM)
        FILNAM = OUTNAM//'.INC'
        CALL SHIFTL (FILNAM)
        CALL SHIFTR (ENVSTR)

C --- open output file
        CALL Findunit (10,INC)
        OPEN (INC,FILE = ENVSTR//FILNAM, STATUS = STASTR, 
     &  ACCESS = 'SEQUENTIAL')

C --- write header
        WRITE (INC,10)
 10     FORMAT ('* Water balance increments (cm/period)',//,
     &'*      Date Day    Rain   Irrig  Interc  Runoff   Transpiration',
     &'     Evaporation Drainage QBottom',/,
     &'*dd/mm/yyyy  nr   gross   gross                     pot     act',
     &'     pot     act      net     net',/,
     &'*<========><==><======><======><======><======><======><======>',
     &'<======><======><=======><======>')

        HINCD = .TRUE.
      ENDIF

C --- Write output .INC file

C --- conversion of daynumber to date
      CALL NRTODA (YEAR,DAYNR,M,D)

C --- write output record
      WRITE (INC,20) D,M,YEAR,NINT(T),IGRAI,IGIRD,IINTC,IRUNO,
     &  IPTRA,IQROT,IPEVA,IEVAP,(IQDRAR+IQDRA),IQBOT
20    FORMAT (I3,'/',I2.2,'/',2I4,8F8.3,F9.3,F8.3)

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE OUTSBA (OUTNAM,SBA,HSBAD,ENVSTR,STASTR,DAYNR,YEAR,
     &   DECTOT,ROTTOT,SAMPRO,SAMCRA,SQBOT,SQSUR,SQRAP,SAMAQ,
     &   SQDRA,T,SOLBAL,SQPREC,SQIRRIG)
C ----------------------------------------------------------------------
C     Date               : 18/08/98                            
C     Purpose            : output of salt balance
C     Subroutines called : SHIFTR,SHIFTL,NRTODA
C     Functions called   : -
C     File usage         : OUTNAM.SBA
C ---------------------------------------------------------------------
      IMPLICIT NONE

C --- global
      INTEGER   SBA,DAYNR,YEAR
      REAL*8    SAMPRO,SAMCRA,SQBOT,SQSUR,SQRAP,SAMAQ,SQDRA,T
      REAL*8    SOLBAL,DECTOT,ROTTOT,SQPREC,SQIRRIG

      CHARACTER OUTNAM*8,ENVSTR*50,STASTR*7

      LOGICAL   HSBAD
C ----------------------------------------------------------------------
C --- local
      INTEGER   D,M
      CHARACTER filnam*12
C ----------------------------------------------------------------------
      IF (.NOT.HSBAD) THEN 

C --- open output file
        CALL SHIFTR (OUTNAM)
        FILNAM = OUTNAM//'.SBA'
        CALL SHIFTL (FILNAM)
        CALL SHIFTR (ENVSTR)
        CALL Findunit (10,SBA)

        OPEN (SBA,FILE=ENVSTR//FILNAM,STATUS=STASTR,
     &  ACCESS='SEQUENTIAL')

C --- write headers
        WRITE (SBA,10)
10    FORMAT (
     &'*DATE       DAY      SQTOP      DECTOT      ROTTOT       SAMPRO',
     &'      SAMCRA       SQBOT       SQDRA       SQRAP       SAMAQ',
     &'       SQSUR     SOLBAL      DATE*',/,
     &'*dd/mm/yyyy  nr      mg/cm2      mg/cm2      mg/cm2      mg/cm2',
     &'      mg/cm2      mg/cm2      mg/cm2      mg/cm2      mg/cm2',
     &'      mg/cm2     mg/cm2 dd/mm/yyyy',/,
     &'*<========><==><==========><==========><==========><==========>',
     &'<==========><==========><==========><==========><==========>',
     &'<==========><=========><=========>')

        HSBAD = .TRUE.
      ENDIF

C --- conversion of daynumber to date
      CALL NRTODA (YEAR,DAYNR,M,D)

C --- Write output record .SBA file

      WRITE(SBA,15) D,M,YEAR,NINT(T),(SQPREC+SQIRRIG),DECTOT,ROTTOT,
     &  SAMPRO,SAMCRA,SQBOT,SQDRA,SQRAP,SAMAQ,SQSUR,SOLBAL,D,M,YEAR
 15   FORMAT(I3,'/',I2.2,'/',2I4,10E12.4,1E11.2,I3,'/',I2.2,'/',I4)

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE OUTTEP (OUTNAM,TEP,HTEPD,ENVSTR,STASTR,DAYNR,YEAR,
     &  NUMNOD,TSoil,T)
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : write soil temperatures to output file
C     Subroutines called : SHIFTR,SHIFTL,NRTODA
C     Functions called   : -
C     File usage         : OUTNAM.TEP
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

      INTEGER   TEP,DAYNR,YEAR,NODE,NUMNOD,D,M
      REAL*8    T
      REAL*8    TSoil(MACP)

      CHARACTER OUTNAM*8,FILNAM*12,ENVSTR*50,STASTR*7

      LOGICAL   HTEPD
C ----------------------------------------------------------------------
      IF (.NOT.HTEPD) THEN

C --- compose filename
        CALL SHIFTR (OUTNAM)
        FILNAM = OUTNAM//'.TEP'
        CALL SHIFTL (FILNAM)
        CALL SHIFTR (ENVSTR)

C --- open output file
        CALL Findunit (10,TEP)
        OPEN (TEP,FILE = ENVSTR//FILNAM,STATUS = STASTR,
     &  ACCESS = 'SEQUENTIAL')

C --- write header
        WRITE (TEP,10)
10      FORMAT (
     &'*DATE      DAY Compartment number------------------------------'
     & ,/,
     &'*dd/mm/yyyy nr      1     6    11    16    21    26    31    36'
     & ,/,
     &'*<========><==><====><====><====><====><====><====><====><====>')
        HTEPD = .TRUE.
      ENDIF
 
C --- conversion of daynumber to date
      CALL NRTODA (YEAR,DAYNR,M,D)

      write (TEP,1010) D,M,YEAR,NINT(T),(TSoil(node),node=1,numnod,5)
1010  format(I3,'/',I2.2,'/',2I4,8F6.1)

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE OUTVAP (SWSOLU,OUTNAM,VAP,HVAPD,ENVSTR,STASTR,DAYNR,
     &  YEAR,NUMNOD,Z,THETA,H,CML,SWTEMP,TSoil,T,Q)
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : write to additional short output file
C     Subroutines called : SHIFTR,SHIFTL,NRTODA
C     Functions called   : -
C     File usage         : OUTNAM.VAP
C ---------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      INTEGER   SWSOLU,VAP,DAYNR,YEAR,NUMNOD,SWTEMP

      REAL*8    Z(MACP),CML(MACP),T
      REAL*8    THETA(MACP),H(MACP),TSoil(MACP),Q(MACP+1)

      CHARACTER OUTNAM*8,ENVSTR*50,STASTR*7

      LOGICAL   HVAPD
C ----------------------------------------------------------------------
C --- local
      INTEGER   d,m,node

      CHARACTER filnam*12
      REAL*8 WFLUX,SFLUX
C ----------------------------------------------------------------------
      IF (.NOT.HVAPD) THEN 

C --- compose filename
        CALL SHIFTR (OUTNAM)
        FILNAM = OUTNAM//'.VAP'
        CALL SHIFTL (FILNAM)
        CALL SHIFTR (ENVSTR)
      
C --- open output file
        CALL Findunit (10,VAP)
        OPEN (VAP,FILE=ENVSTR//FILNAM, STATUS = STASTR,
     &  ACCESS='SEQUENTIAL')

C --- write header
        WRITE (VAP,50) 

 50   FORMAT ('*      date       depth  water       head     solute  ',
     &        'temp       Wflux       Sflux',/,
     &        '*                    cm    (-)         cm     mg/cm3  ',
     &        '   C        cm/d    mg/cm2/d')


c        HVAPD = .TRUE.

      ENDIF

C --- Write output .VAP file

C --- conversion of daynumber to date
      CALL NRTODA (YEAR,DAYNR,M,D)

C --- write output record

      DO 20 NODE = 1,NUMNOD
        if (SWTEMP.EQ.0) TSoil(NODE) = -99.9d0
        if (SWSOLU.EQ.0) CML(NODE)  = 0.0
        WFLUX = (Q(NODE)+Q(Node+1))/2
        SFLUX = WFLUX*CML(NODE)

        IF  (.not. HVAPD) then
          WRITE (VAP,79) D,M,YEAR,NINT(T),Z(NODE),THETA(NODE),
     &                   H(NODE),CML(NODE), TSoil(NODE),WFLUX,SFLUX
        ELSE
          WRITE (VAP,80) D,M,YEAR,NINT(T),Z(NODE),THETA(NODE),
     &                   H(NODE),CML(NODE), TSoil(NODE),WFLUX,SFLUX
        ENDIF

20    CONTINUE
      HVAPD = .true.

      WRITE (VAP,'(2X)')

 79   FORMAT('*',I2,'/',I2.2,'/',2I4,1X,F7.1,F7.3,F11.3,E11.3,F6.1,
     & 2E12.3)
 80   FORMAT(    I3,'/',I2.2,'/',2I4,1X,F7.1,F7.3,F11.3,E11.3,F6.1,
     & 2E12.3)

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE OUTWBA (OUTNAM,WBA,HWBAD,ENVSTR,STASTR,DAYNR,YEAR,
     &  CGRAI,CNRAI,CGIRD,CNIRD,CRUNO,CQROT,CEVAP,CQBOT,CQDRA,
     &  CPTRA,CPEVA,GWL,T,CQDRAR,VOLACT,VOLINI,POND,RUNNR,
     &  SWSCRE,CQDARCY,CQTOP,TCUM)
C ----------------------------------------------------------------------
C     Date               : 21/9/99
C     Purpose            : write water balance data to OUTNAM.WBA file
C     Subroutines called : SHIFTR,SHIFTL,NRTODA
C     Functions called   : -
C     File usage         : OUTNAM.WBA
C ---------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER   WBA,DAYNR,YEAR,D,M,RUNNR,SWSCRE

      REAL*8    CEVAP,CGIRD,CGRAI,CNIRD,CNRAI,CPEVA,CPTRA,CQBOT,DSTOR
      REAL*8    CQDRA,CQROT,CRUNO,GWL,T,CQDRAR,POND,CQDARCY,CQTOP,TCUM
      REAL*8    VOLACT,VOLINI

      CHARACTER OUTNAM*8,FILNAM*12,ENVSTR*50,STASTR*7

      LOGICAL   HWBAD
C ----------------------------------------------------------------------
      IF (.NOT.HWBAD) THEN

C --- compose filename
        CALL SHIFTR (OUTNAM)
        FILNAM = OUTNAM//'.WBA'
        CALL SHIFTL (FILNAM)
        CALL SHIFTR (ENVSTR)

C --- open WBA output file
        CALL Findunit (10,WBA)
        OPEN (WBA,FILE = ENVSTR//FILNAM, STATUS = STASTR, 
     &  ACCESS = 'SEQUENTIAL')

C --- write header WBA file
        WRITE (WBA,10)
10      FORMAT (
     &'*      DATE DAY   RAIN [cm]     IRR [cm]      RUO    TRA [cm]  ',
     &'    EVS [cm]      FLUX [cm]      DSTOR     GWL  QDIF      Date*',
     & /, 
     &'*dd/mm/yyyy  nr    gro....net   gro...net      cm    pot....act',
     &'    pot....act     lat.....bot      cm      cm    cm dd/mm/yyyy',
     & /,
     &'*<========><==><=====><=====><====><====><======><=====><=====>',
     &'<=====><=====><======><======><======><======><====><=========>')

C --- write header screen output
        IF (SWSCRE.EQ.1) THEN
          WRITE (*,20) RUNNR
 20       FORMAT(/,' RUN ',I2,T20,
     &           'Cumulative water balance components (cm)',/)
          WRITE (*,22)
 22       FORMAT ('       Date   Rain  Irrig Runoff Transp Evapor',
     &            '  Drain   Qbot    Gwl   Qdif'/,
     &            '             gross  gross        actual actual',
     &            '    net    net           cum')
        endif
        HWBAD = .TRUE.
      ENDIF
C --- write output water balance components

C --- conversion of daynumber to date
      CALL NRTODA (YEAR,DAYNR,M,D)

C --- write output record WBA file
      DSTOR = REAL(VOLACT) + POND - REAL(VOLINI)
      WRITE (WBA,30) D,M,YEAR,NINT(T),CGRAI,CNRAI,CGIRD,CNIRD,CRUNO,
     &  CPTRA,CQROT,CPEVA,CEVAP,(CQDRA+CQDRAR),CQBOT,DSTOR,GWL,
     &  (CQDARCY-CQTOP),D,M,YEAR
 30   FORMAT (I3,'/',I2.2,'/',2I4,2(1x,F6.2),2(1x,F5.1),1x,F7.2,
     &        4(1x,F6.2),3(1x,F7.2),1x,F7.1,1x,F5.2,I3,'/',I2.2,'/',I4)

C --- write output record screen
      IF (SWSCRE.EQ.1) THEN
        WRITE (*,40) D,M,YEAR,CGRAI,CGIRD,CRUNO,CQROT,CEVAP,
     &               (CQDRA+CQDRAR),CQBOT,GWL,(CQDARCY-CQTOP)
 40     FORMAT(I3,'/',I2.2,'/',I4,7F7.2,F7.1,F7.2)
      elseif (SWSCRE.EQ.2) then
        Write (*,'(1H+,''            '', i4,3X,i3,1X,i5)')
     &   year,daynr,INT(TCUM)

      ENDIF

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE PENMON (DAYNR,LAT,ALT,A,B,RCS,RCC,RAD,TAV,HUM,
     & WIN,RSC,ES0,ET0,EW0,SWCF,CF,ISTAGE)
C ----------------------------------------------------------------------
C     Date               : 14/01/99
C     Purpose            : Calculation of potential evaporation & 
C                          transpiration rates from a bare soil surface,
C                          a dry and a wet crop canopy;           
C                          based on the Penman - Monteith approach.
C
C     Formal parameters  : (I = input, O = output)
C     DAYNR  daynumber (January 1st = 1) [-]...........................I
C     LAT    latitude [degr.,dec.degr., N=+,S=-].......................I
C     ALT    altitude above mean sea level [m].........................I
C     A      first  Angstrom coefficient [-]...........................I
C     B      second Angstrom coefficient [-]...........................I
C     RCS    reflection coefficient soil [-]...........................I
C     RCC    reflection coefficient crop [-]...........................I
C     RAD    incoming short wave radiation [J/m2/d]....................I
C     TAV    average day temperature [C]...............................I
C     HUM    vapour pressure [kPa].....................................I
C     WIN    windspeed at 2 m height [m/s].............................I
C     RCS    minimum canopy resistance [s/m]...........................I
C     ES0    potential evaporation rate from a  bare soil [mm/d].......O
C     ET0    potential transpiration rate from a dry crop [mm/d].......O
C     EW0    potential transpiration rate from a wet crop [mm/d].......O
C     SWCF   CF contains crop factor (=1) or crop height (=2)
C     CF     crop factor [-] or crop height [cm]
C     IStage no crop [=1] or yes crop [=2]
C
C     Subroutines called : ASTRO
C
C     Functions called   : LIMIT
C
C     File usage         : -
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER DAYNR,SWCF,IStage

      REAL*8  LIMIT,LAMBDA,LAT,CHRef,CF
      REAL*8  A,ALT,ATMTR,B,COSLD,CP,DAYL,DAYLP,DELTA,DSINB,DSO,EA
      REAL*8  ED,ES0,ET0,ETAERC,ETAERS,ETAERW,ETRADC,ETRADS,ETRADW,EW0
      REAL*8  G,GAMMA,GAMMOC,GAMMOS,GAMMOW,HUM,PALT,PI,RAC,RAD,RAS,RAW
      REAL*8  RCC,RCS,RELSSD,RHO,RNC,RNL,RNS,RNW,RSC,RSS,RSW,SC,SINLD
      REAL*8  TAV,TAVK,TKV,UD,VPD,WIN,CH

      PARAMETER (CHRef = 12.0d0, PI = 3.1415962d0)
C ----------------------------------------------------------------------
C --- avoid zero crop height and crop height > 3.0 m
      If (Istage.EQ.1) then
        CH = CHref
      else 
        If (SWCF.EQ.1) Then
          CH = CHRef
        Else
          CH = MAX (CF,0.1d0)
          CH = Min (CF,295.0d0)
        Endif
      Endif
     
C --- conversion of average temperature from [C] to [K]
      TAVK = TAV+273.0d0

C --- atmospheric pressure at elevation ALT [kPa]
      PALT = 101.3*((TAVK-0.0065*ALT)/TAVK)**5.26

C --- latent heat of vaporization [MJ/kg]
      LAMBDA = 2.501d0-0.002361*TAV

C --- saturation vapour pressure at TAV [kPa]
      EA  = 0.611*EXP(17.27*TAV/(TAV+237.3d0))
C --- measured vapour pressure not to exceed saturated vapour pressure
      ED  = MIN(HUM,EA)
C --- vapour pressure deficit [kPa]
      VPD = EA-ED

C --- slope vapour pressure curve [kPa/C]
      DELTA = 4098*EA/(TAV+237.3d0)**2

C --- psychrometric constant [kPa/C]
      GAMMA = 0.00163*PALT/LAMBDA

C --- atmospheric density [kg/m3]
      TKV = TAVK/(1.0d0-0.378*ED/PALT)
      RHO = 3.486*PALT/TKV

C --- specific heat moist air [kJ/kg/C]
      CP = 622*GAMMA*LAMBDA/PALT

C --- day wind [m/s], avoid zero windspeed
      UD = MAX (WIN*1.33,0.0001d0)

C --- surface resistance [s/m] - Soil & Wet crop
      RSS = 0
      RSW = 0

C --- aerodynamic resistance [s/m] - Soil, Crop & Wet crop
      RAS = LOG((200.0d0-0.67*0.1d0)/0.123/0.1)*LOG((200.0d0-0.67*0.1d0)
     &      /0.0123/0.1)/(0.1681*UD)
      RAC = LOG((200.0d0-0.67*CH)/0.123/CH)*LOG((200.0d0-0.67*CH)/
     &      0.0123/CH)/(0.1681*UD)
      RAW = LOG((200.0d0-0.67*CH)/0.123/CH)*LOG((200.0d0-0.67*CH)/
     &      0.0123/CH)/(0.1681*UD)

C --- modified psychrometric constant [kPa/C] - Soil, Crop & Wet crop 
      GAMMOS = GAMMA*(1.0d0+RSS/RAS)
      GAMMOC = GAMMA*(1.0d0+RSC/RAC)
      GAMMOW = GAMMA*(1.0d0+RSW/RAW)

C --- net short wave radiation [MJ/m**2/d] - Soil, Crop & Wet crop
      RNS = (1.0d0-RCS)*RAD/1000000
      RNC = (1.0d0-RCC)*RAD/1000000
      RNW = (1.0d0-RCC)*RAD/1000000

C --- net long wave radiation [MJ/m**2/d]
      CALL ASTRO (DAYNR,LAT,DAYL,DAYLP,SINLD,COSLD)
      DSINB  = 3600.*(DAYL*SINLD+24.*COSLD*SQRT(1.0d0-
     &         (SINLD/COSLD)**2)/PI)
      SC     = 1370.*(1.0d0+0.033*COS(2.*PI*DAYNR/365.))
      DSO    = SC*DSINB
      ATMTR  = RAD/DSO
      RELSSD = LIMIT(0.0d0,1.0d0,(ATMTR-A)/B)
      RNL    = 4.9d-9*TAVK**4*(0.34d0-0.14*SQRT(ED))*(0.1d0+0.9*RELSSD)

C --- soil heat flux [MJ/m2/d]
      G = 0.0

C --- aerodynamic term of the PM equation [mm/d] - Soil, Crop & Wet crop
      ETAERS = (86.4/LAMBDA)*(1/(DELTA+GAMMOS))*(RHO*CP*VPD/RAS)
      ETAERC = (86.4/LAMBDA)*(1/(DELTA+GAMMOC))*(RHO*CP*VPD/RAC)
      ETAERW = (86.4/LAMBDA)*(1/(DELTA+GAMMOW))*(RHO*CP*VPD/RAW)

C --- radiation term of the PM equation [mm/d] - Soil, Crop & Wet crop
      ETRADS = DELTA/(DELTA+GAMMOS)*(RNS-RNL-G)*1/LAMBDA      
      ETRADC = DELTA/(DELTA+GAMMOC)*(RNC-RNL-G)*1/LAMBDA      
      ETRADW = DELTA/(DELTA+GAMMOW)*(RNW-RNL-G)*1/LAMBDA      

C --- sum of both terms [mm/d] - Soil, Crop and Wet crop      
      ES0 = MAX (0.0d0,ETAERS+ETRADS)
      ET0 = MAX (0.0d0,ETAERC+ETRADC)
      EW0 = MAX (0.0d0,ETAERW+ETRADW)

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE RADIAT (IDAY,HOUR,DAYL,SINLD,COSLD,AVRAD,SINB,
     $                   PARDIR,PARDIF)
C ----------------------------------------------------------------------
C --- Author: Daniel van Kraalingen, 1986
C --- Calculates the fluxes of diffuse and direct photosynthetically
C --- active radiation from the total daily shortwave radiation actually
C --- received (AVRAD) for a given day of the year and hour of the day.
C --- The input variables DAYL, SINLD and COSLD are calculated in ASTRO.
C --- For more information: see Spitters et al. (1988).
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER IDAY

      REAL*8  AOB,ATMTR,AVRAD,COSLD,DAYL,DSINB,DSINBE,DSO,FRDIF,HOUR
      REAL*8  PAR,PARDIF,PARDIR,PI,SC,SINB,SINLD

      DATA    PI /3.1415926d0/
C ----------------------------------------------------------------------
C --- calculations on solar elevation
C --- sine of solar elevation SINB
      AOB    = SINLD/COSLD
      SINB   = MAX (0.0d0,SINLD+COSLD*COS(2.0*PI*(HOUR+12.0d0)/24.0))
C --- integral of SINB
      DSINB  = 3600.*(DAYL*SINLD+24.*COSLD*SQRT(1.0d0-AOB*AOB)/PI)
C --- integral of SINB, corrected for lower atmospheric transmission
C --- at low solar elevations
      DSINBE = 3600.*(DAYL*(SINLD+0.4*(SINLD*SINLD+COSLD*COSLD*0.5))+
     $         12.0*COSLD*(2.0d0+3.0*0.4*SINLD)*SQRT(1.0d0-AOB*AOB)/PI)

C --- solar constant and daily extraterrestrial radiation
      SC     = 1370.*(1.0d0+0.033*COS(2.0*PI*IDAY/365.))
      DSO    = SC*DSINB

C --- diffuse light fraction from atmospheric transmission
      ATMTR  = AVRAD/DSO
      IF (ATMTR.GT.0.75d0) FRDIF = 0.23d0
      IF (ATMTR.LE.0.75d0.AND.ATMTR.GT.0.35d0) FRDIF = 1.33d0-1.46*ATMTR
      IF (ATMTR.LE.0.35d0.AND.ATMTR.GT.0.07d0) 
     $ FRDIF = 1.0d0-2.3*(ATMTR-0.07d0)**2
      IF (ATMTR.LE.0.07d0) FRDIF = 1.0d0

C --- photosynthetic active radiation, diffuse and direct
      PAR    = 0.5*AVRAD*SINB*(1.0d0+0.4*SINB)/DSINBE
      PARDIF = MIN (PAR,SINB*FRDIF*ATMTR*0.5*SC)
      PARDIR = PAR-PARDIF

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE RDBBC (BBCFIL,ENVSTR,BAYRD,BAYRY,GWLTAB,SHAPE,RIMLAY,
     & AQAVE,AQAMP,AQTAMX,AQOMEG,QBOTAB,COFQHA,COFQHB,HGITAB,SWBOTB,
     & LOGF,YEAR,FMAY,HDRAIN)
C ----------------------------------------------------------------------
C     Date               : 17/4/98                                
C     Purpose            : reading b.c.'s at bottom of soil profile,   
C                          for the entire calculation period.          
C     Subroutines called : DATONR,SHIFTL,SHIFTR            
C     Functions called   : FROMTO                                      
C     File usage         : .BBC, file containing bottom boundary data  
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER   BBC,FROMTO,BAYRD,BAYRY,SWBOTB,NR,LOGF,YEAR,FMAY

      REAL*8    GWLTAB(732),QBOTAB(732),HGITAB(50)
      REAL*8    SHAPE,RIMLAY,AQAVE,AQAMP,AQTAMX,AQOMEG,HDRAIN
      REAL*8    AQPER,COFQHA,COFQHB,PI

      CHARACTER BBCFIL*8,ENVSTR*50,FILNAM*12

      PARAMETER (PI=3.14159d0)
C ----------------------------------------------------------------------
C --- local
      INTEGER SWOPT,SWC2,dd(366),mm(366),yy(366),I,IFND

      REAL*8  value(366),C2AVE,C2AMP,C2MAX

      CHARACTER LINE*80
      LOGICAL BLANK,COMMENT,EXISTS
C ----------------------------------------------------------------------

C --- compose filename
      CALL SHIFTR (BBCFIL)
      FILNAM = BBCFIL//'.BBC'
      CALL SHIFTL (FILNAM)
      CALL SHIFTR (ENVSTR)

C --- check if a file with bottom boundary conditions does exist
      INQUIRE (FILE=ENVSTR//FILNAM,EXIST=EXISTS)

      IF (.NOT.EXISTS) THEN
C ---   write error to log file
        WRITE (LOGF,98) ENVSTR//FILNAM
 98     FORMAT ('SWAP can not find the input file',/,A)
        STOP 'The .BBC file was not found: SWAP stopped !'
      ENDIF

C --- open file with bottom boundary data
      CALL FINDUNIT (14,BBC)
      OPEN (BBC,FILE=ENVSTR//FILNAM,status='old',access='sequential')

C --- given groundwaterlevel
      CALL RSINTR (BBC,FILNAM,LOGF,'SWOPT1',0,1,SWOPT)
      IF (SWOPT.EQ.1) THEN

C ---   position file pointer
        IFND = 0
14      READ (BBC,'(A)') LINE
        CALL ANALIN (LINE,BLANK,COMMENT)
        IF (BLANK .OR. (COMMENT .AND. IFND .EQ. 0)) GOTO 14
        IF (COMMENT) GOTO 15
        IFND = IFND+1
        BACKSPACE (BBC)
        READ (BBC,*) DD(IFND),MM(IFND),value(IFND)
        GOTO 14
C ---   ready...
15      CONTINUE
C ---   year is calculated
        DO 16 I = 1,IFND 
          if (mm(I).GE.FMAY) yy(I) = BAYRY
          if (mm(I).LT.FMAY) yy(I) = BAYRY+1
16      CONTINUE
        DO 17, I=1,IFND
          CALL DATONR (yy(I),mm(I),dd(I),nr)
          GWLTAB(I*2)   = value(I) 
          GWLTAB(I*2-1) = FROMTO (BAYRD,BAYRY,NR,YY(I))+1
17      CONTINUE
        SWBOTB = 1
        GOTO 1000
      ENDIF 
      CALL JMPLBL (BBC,FILNAM,LOGF,'SWOPT2')

C --- regional bottom flux is given                             
      CALL RSINTR (BBC,FILNAM,LOGF,'SWOPT2',0,1,SWOPT)
      IF (SWOPT.EQ.1) THEN
        CALL RSINTR (BBC,FILNAM,LOGF,'SWC2',1,2,SWC2)
        IF (SWC2 .EQ. 1) THEN
          CALL RSDOUR (BBC,FILNAM,LOGF,'C2AVE',-10.0,10.0,C2AVE)
          CALL RSDOUR (BBC,FILNAM,LOGF,'C2AMP',-10.0,10.0,C2AMP)
          CALL RSDOUR (BBC,FILNAM,LOGF,'C2MAX',1.0,366.0,C2MAX)
          C2MAX = FROMTO (BAYRD,BAYRY,INT(C2MAX),YEAR)+1
C ---     fill table with sine function
          DO 19 I = 1,366
            QBOTAB(I*2)   = C2AVE+C2AMP*cos(0.0172*(I-C2MAX)) 
            QBOTAB(I*2-1) = I
 19       CONTINUE 
          SWBOTB = 2
          GOTO 1000
        ELSE
C ---     read table, position pointer
          Call JMPLBL (BBC,FILNAM,LOGF,'C2MAX')
          Read (BBC,'(A)') LINE
          IFND = 0
24        READ (BBC,'(A)') LINE
          CALL ANALIN (LINE,BLANK,COMMENT)
          IF (BLANK .OR. (COMMENT .AND. IFND .EQ. 0)) GOTO 24
          IF (COMMENT) GOTO 25
          IFND = IFND+1
          BACKSPACE (BBC)
          READ (BBC,*) DD(IFND),MM(IFND),value(IFND)
          GOTO 24
C ---     ready...
25        CONTINUE
C ---     year is calculated
          DO 26 I = 1,IFND 
            if (mm(I).GE.FMAY) yy(I) = BAYRY
            if (mm(I).LT.FMAY) yy(I) = BAYRY+1
26        CONTINUE
          DO 27, I=1,IFND
            CALL DATONR (yy(I),mm(I),dd(I),nr)
            QBOTAB(I*2)   = value(I) 
            QBOTAB(I*2-1) = FROMTO (BAYRD,BAYRY,NR,YY(I))+1
27        CONTINUE
          SWBOTB = 2
          GOTO 1000
        ENDIF
      ENDIF 
      CALL JMPLBL (BBC,FILNAM,LOGF,'SWOPT3')

C --- calculated flux through the bottom of the profile
      CALL RSINTR (BBC,FILNAM,LOGF,'SWOPT3',0,1,SWOPT)
      IF (SWOPT.EQ.1) THEN
        CALL RSDOUR (BBC,FILNAM,LOGF,'SHAPE',0.0,1.0,SHAPE)
        CALL RSDOUR (BBC,FILNAM,LOGF,'HDRAIN',-1.0E4,0.0,HDRAIN)
        CALL RSDOUR (BBC,FILNAM,LOGF,'RIMLAY',0.0,1.0E5,RIMLAY)
        CALL RSDOUR (BBC,FILNAM,LOGF,'AQAVE',-1000.0,1000.0,AQAVE)
        CALL RSDOUR (BBC,FILNAM,LOGF,'AQAMP',0.0,1000.0, AQAMP)
        CALL RSINTR (BBC,FILNAM,LOGF,'AQTAMX',1,366,NR)
        AQTAMX = FROMTO (BAYRD,BAYRY,NR,YEAR)+1
        CALL RSDOUR (BBC,FILNAM,LOGF,'AQPER',0.0,366.0,AQPER)
        AQOMEG = PI*2./365.
        IF (AQAMP.GT.0.1d0) THEN
          AQOMEG = PI*2./AQPER
        ENDIF
        SWBOTB = 3
        GOTO 1000
      ENDIF
      CALL JMPLBL (BBC,FILNAM,LOGF,'SWOPT4')

C --- flux-groundwater level relationship 
      CALL RSINTR (BBC,FILNAM,LOGF,'SWOPT4',0,1,SWOPT)
      IF (SWOPT.EQ.1) THEN
        CALL RSDOUR (BBC,FILNAM,LOGF,'COFQHA',-100.0,100.0,COFQHA)
        CALL RSDOUR (BBC,FILNAM,LOGF,'COFQHB',-1.0,1.0,COFQHB)
        SWBOTB = 4
        GOTO 1000
      ENDIF
      CALL JMPLBL (BBC,FILNAM,LOGF,'SWOPT5')

C --- pressure head of lowest compartment is given 
      CALL RSINTR (BBC,FILNAM,LOGF,'SWOPT5',0,1,SWOPT)
      IF (SWOPT.EQ.1) THEN

C ---   position file pointer
        IFND = 0
34      READ (BBC,'(A)') LINE
        CALL ANALIN (LINE,BLANK,COMMENT)
        IF (BLANK .OR. (COMMENT .AND. IFND .EQ. 0)) GOTO 34
        IF (COMMENT) GOTO 35
        IFND = IFND+1
        BACKSPACE (BBC)
        READ (BBC,*) DD(IFND),MM(IFND),value(IFND)
        GOTO 34
C ---   ready...
35      CONTINUE
C ---   year is calculated
        DO 36 I = 1,IFND 
          if (mm(I).GE.FMAY) yy(I) = BAYRY
          if (mm(I).LT.FMAY) yy(I) = BAYRY+1
36      CONTINUE
        DO 37, I=1,IFND
          CALL DATONR (yy(I),mm(I),dd(I),nr)
          HGITAB(I*2)   = value(I) 
          HGITAB(I*2-1) = FROMTO (BAYRD,BAYRY,NR,YY(I))+1
37      CONTINUE
        SWBOTB = 5
        GOTO 1000
      ENDIF 
      CALL JMPLBL (BBC,FILNAM,LOGF,'SWOPT6')

C --- zero flux
      CALL RSINTR (BBC,FILNAM,LOGF,'SWOPT6',0,1,SWOPT)
      IF (SWOPT.EQ.1) THEN
        SWBOTB = 6
        GOTO 1000
      ENDIF
      CALL JMPLBL (BBC,FILNAM,LOGF,'SWOPT7')

C --- free drainage 
      CALL RSINTR (BBC,FILNAM,LOGF,'SWOPT7',0,1,SWOPT)
      IF (SWOPT.EQ.1) THEN
        SWBOTB = 7
        GOTO 1000
      ENDIF
      CALL JMPLBL (BBC,FILNAM,LOGF,'SWOPT8')

C --- lysimeter with free outflow 
      CALL RSINTR (BBC,FILNAM,LOGF,'SWOPT8',0,1,SWOPT)
      IF (SWOPT.EQ.1) THEN
        SWBOTB = 8
        GOTO 1000
      ENDIF

1000  CLOSE (BBC)
      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE RDCRPD (CRPFIL,ENVSTR,SWCF,CFTB,IDSL,DLO,DLC,TSUMEA,
     &  TSUMAM,DTSMTB,DVSEND,TDWI,LAIEM,RGRLAI,SLATB,SPA,
     &  SSA,SPAN,TBASE,KDIF,KDIR,EFF,AMAXTB,TMPFTB,TMNFTB,CVL,CVO,CVR,
     &  CVS,Q10,RML,RMO,RMR,RMS,RFSETB,FRTB,FLTB,FSTB,FOTB,PERDL,RDRRTB,
     &  RDRSTB,HLIM1,HLIM2U,HLIM2L,HLIM3H,HLIM3L,HLIM4,RSC,ADCRH,
     &  ADCRL,ECMAX,ECSLOP,COFAB,RDI,RRI,RDC,RDCTB,LOGF)
C ----------------------------------------------------------------------
C     Date               : 14/1/99    
C     Purpose            : get crop parameters from cropfile
C     Subroutines called : SHIFTR, SHIFTL
C     Functions called   : -
C     File usage         : CRPFIL, file containing crop parameters
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER CRP,I,IDSL,LOGF,IFND,SWCF
 
      REAL*8 CFTB(30),DTSMTB(30),SLATB(30),AMAXTB(30),TMPFTB(30)
      REAL*8 TMNFTB(30),RFSETB(30),FRTB(30),FLTB(30),FSTB(30),FOTB(30)
      REAL*8 ECMAX,ECSLOP,RDRRTB(30),RDRSTB(30),KDIF,KDIR,COFAB,LAIEM
      REAL*8 ADCRH,ADCRL,CVL,CVO,CVR,CVS,DLC,DLO,DVSEND,EFF
      REAL*8 PERDL,Q10,RDC,RDI,RGRLAI,RML,RMO,RMR,RMS,RRI,SPA,SPAN
      REAL*8 SSA,TBASE,TDWI,TSUMAM,TSUMEA,RDCTB(22)
      REAL*8 HLIM1,HLIM2U,HLIM2L,HLIM3H,HLIM3L,HLIM4,RSC
      REAL*8 X,SUM,RFAC,VALUE,AFGEN

      CHARACTER CRPFIL*8,ENVSTR*50,FILNAM*12

      LOGICAL   EXISTS
C ----------------------------------------------------------------------
C --- compose filename 
      CALL SHIFTR (CRPFIL)
      FILNAM = CRPFIL//'.CRP'
      CALL SHIFTL (FILNAM)
      CALL SHIFTR (ENVSTR)

C --- check if a file with detailed crop data does exist
      INQUIRE (FILE=ENVSTR//FILNAM,EXIST=EXISTS)

      IF (.NOT.EXISTS) THEN
C ---   write error to log file
        WRITE (LOGF,92) ENVSTR//FILNAM
 92     FORMAT ('SWAP can not find the input file',/,A)
        STOP 'The .CRP file was not found: SWAP stopped !'
      ENDIF

C --- open file with crop data
      Call Findunit (10,CRP) 
      OPEN (CRP,FILE=ENVSTR//FILNAM,STATUS='OLD',
     & access='sequential')

C --- Crop factor or crop height
      CALL RSINTR (CRP,FILNAM,LOGF,'SWCF',1,2,SWCF)
      IF (SWCF.EQ.1) then
        CALL RADOUR (CRP,FILNAM,LOGF,'CFTB',0.0,2.01,CFTB,30,IFND)
      ELSEIF (SWCF.EQ.2) then
        CALL RADOUR (CRP,FILNAM,LOGF,'CFTB',0.0,1000.0,CFTB,30,IFND)
      ENDIF

C --- phenology
      CALL RSINTR (CRP,FILNAM,LOGF,'IDSL',0,2,IDSL)
      IF (IDSL.EQ.1.OR.IDSL.EQ.2) then
        CALL RSDOUR (CRP,FILNAM,LOGF,'DLO',0.0,24.0,DLO)
        CALL RSDOUR (CRP,FILNAM,LOGF,'DLC',0.0,24.0,DLC)
      ENDIF
      CALL JMPLBL (CRP,FILNAM,LOGF,'TSUMEA')
      IF (IDSL.EQ.0.OR.IDSL.EQ.2) then
        CALL RSDOUR (CRP,FILNAM,LOGF,'TSUMEA',0.0,10000.0,TSUMEA)
        CALL RSDOUR (CRP,FILNAM,LOGF,'TSUMAM',0.0,10000.0,TSUMAM)
      ENDIF
      CALL JMPLBL (CRP,FILNAM,LOGF,'DTSMTB')
      CALL RADOUR (CRP,FILNAM,LOGF,'DTSMTB',0.0,100.0,DTSMTB,30,IFND) 
      CALL RSDOUR (CRP,FILNAM,LOGF,'DVSEND',0.0,3.0,DVSEND)

C --- initial
      CALL RSDOUR (CRP,FILNAM,LOGF,'TDWI',0.0,10000.0,TDWI)
      CALL RSDOUR (CRP,FILNAM,LOGF,'LAIEM',0.0,10.0,LAIEM)
      CALL RSDOUR (CRP,FILNAM,LOGF,'RGRLAI',0.0,1.0,RGRLAI)

C --- green area
      CALL RADOUR (CRP,FILNAM,LOGF,'SLATB',0.0,2.0,SLATB,30,IFND)
      CALL RSDOUR (CRP,FILNAM,LOGF,'SPA',0.0,1.0,SPA)
      CALL RSDOUR (CRP,FILNAM,LOGF,'SSA',0.0,1.0,SSA)
      CALL RSDOUR (CRP,FILNAM,LOGF,'SPAN',0.0,366.0,SPAN)
      CALL RSDOUR (CRP,FILNAM,LOGF,'TBASE',-10.0,30.0,TBASE)

C --- assimilation
      CALL RSDOUR (CRP,FILNAM,LOGF,'KDIF',0.0,2.0,KDIF)
      CALL RSDOUR (CRP,FILNAM,LOGF,'KDIR',0.0,2.0,KDIR)
      CALL RSDOUR (CRP,FILNAM,LOGF,'EFF',0.0,10.0,EFF)
      CALL RADOUR (CRP,FILNAM,LOGF,'AMAXTB',0.0,100.0,AMAXTB,30,IFND)
      CALL RADOUR (CRP,FILNAM,LOGF,'TMPFTB',-10.0,50.0,TMPFTB,30,IFND)
      CALL RADOUR (CRP,FILNAM,LOGF,'TMNFTB',-10.0,50.0,TMNFTB,30,IFND)

C --- conversion of assimilates into biomass
      CALL RSDOUR (CRP,FILNAM,LOGF,'CVL',0.0,1.0,CVL)
      CALL RSDOUR (CRP,FILNAM,LOGF,'CVO',0.0,1.0,CVO)
      CALL RSDOUR (CRP,FILNAM,LOGF,'CVR',0.0,1.0,CVR)
      CALL RSDOUR (CRP,FILNAM,LOGF,'CVS',0.0,1.0,CVS)

C --- maintenance respiration
      CALL RSDOUR (CRP,FILNAM,LOGF,'Q10',0.0,5.0,Q10)
      CALL RSDOUR (CRP,FILNAM,LOGF,'RML',0.0,1.0,RML)
      CALL RSDOUR (CRP,FILNAM,LOGF,'RMO',0.0,1.0,RMO)
      CALL RSDOUR (CRP,FILNAM,LOGF,'RMR',0.0,1.0,RMR)
      CALL RSDOUR (CRP,FILNAM,LOGF,'RMS',0.0,1.0,RMS)
      CALL RADOUR (CRP,FILNAM,LOGF,'RFSETB',0.0,3.0,RFSETB,30,IFND)

C --- partitioning
      CALL RADOUR (CRP,FILNAM,LOGF,'FRTB',0.0,3.0,FRTB,30,IFND)
      CALL RADOUR (CRP,FILNAM,LOGF,'FLTB',0.0,3.0,FLTB,30,IFND)
      CALL RADOUR (CRP,FILNAM,LOGF,'FSTB',0.0,3.0,FSTB,30,IFND)
      CALL RADOUR (CRP,FILNAM,LOGF,'FOTB',0.0,3.0,FOTB,30,IFND)

C --- death rates
      CALL RSDOUR (CRP,FILNAM,LOGF,'PERDL',0.0,3.0,PERDL)
      CALL RADOUR (CRP,FILNAM,LOGF,'RDRRTB',0.0,3.0,RDRRTB,30,IFND)
      CALL RADOUR (CRP,FILNAM,LOGF,'RDRSTB',0.0,3.0,RDRSTB,30,IFND)

C --- water use
      CALL RSDOUR (CRP,FILNAM,LOGF,'HLIM1' ,-100.0,100.0,HLIM1)
      CALL RSDOUR (CRP,FILNAM,LOGF,'HLIM2U',-1000.0,100.0,HLIM2U)
      CALL RSDOUR (CRP,FILNAM,LOGF,'HLIM2L',-1000.0,100.0,HLIM2L)
      CALL RSDOUR (CRP,FILNAM,LOGF,'HLIM3H',-10000.0,100.0,HLIM3H)
      CALL RSDOUR (CRP,FILNAM,LOGF,'HLIM3L',-10000.0,100.0,HLIM3L)
      CALL RSDOUR (CRP,FILNAM,LOGF,'HLIM4' ,-16000.0,100.0,HLIM4)
      CALL RSDOUR (CRP,FILNAM,LOGF,'RSC',0.0,1000.0,RSC)
      CALL RSDOUR (CRP,FILNAM,LOGF,'ADCRH',0.0,5.0,ADCRH)
      CALL RSDOUR (CRP,FILNAM,LOGF,'ADCRL',0.0,5.0,ADCRL)
      CALL RSDOUR (CRP,FILNAM,LOGF,'ECMAX',0.0,20.0,ECMAX)
      CALL RSDOUR (CRP,FILNAM,LOGF,'ECSLOP',0.0,40.0,ECSLOP)
      CALL RSDOUR (CRP,FILNAM,LOGF,'COFAB',0.0,1.0,COFAB)

C --- rooting
      CALL RADOUR (CRP,FILNAM,LOGF,'RDCTB',0.0,100.0,RDCTB,22,IFND)
      CALL RSDOUR (CRP,FILNAM,LOGF,'RDI',0.0,1000.0,RDI)
      CALL RSDOUR (CRP,FILNAM,LOGF,'RRI',0.0,100.0,RRI)
      CALL RSDOUR (CRP,FILNAM,LOGF,'RDC',0.0,1000.0,RDC)

C --- normalize RDCTB
      SUM = 0.0
      DO 96 X = 0.05d0,1.0d0,0.1d0
        VALUE = AFGEN (RDCTB,22,X)
        SUM = SUM+0.1*VALUE
96    CONTINUE

      RFAC = 1/SUM
      DO 98 I = 1,IFND/2
98    RDCTB(I*2) = RDCTB(I*2)*RFAC

      CLOSE (CRP)

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE RDCRPS (CRPFIL,ENVSTR,IDEV,LCC,TSUMEA,TSUMAM,TBASE,
     & KDIF,KDIR,GCTB,SWGC,CFTB,SWCF,RDTB,KYTB,HLIM1,HLIM2U,HLIM2L,
     & HLIM3H,HLIM3L,HLIM4,RSC,ADCRH,ADCRL,ECMAX,ECSLOP,RDCTB,
     & COFAB,LOGF)
C ----------------------------------------------------------------------
C     Date               : 21/1/99             
C     Purpose            : get crop parameters from cropfile
C     Subroutines called : SHIFTR, SHIFTL
C     Functions called   : AFGEN
C     File usage         : CRPFIL, file containing crop parameters
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER   CRP,IDEV,I,LCC,SWGC,SWCF,LOGF,IFND

      REAL*8    GCTB(72),CFTB(72),RDTB(72),KYTB(72),RDCTB(22)
      REAL*8    ADCRH,ADCRL,ECMAX,ECSLOP,TBASE,TSUMAM,TSUMEA
      REAL*8    HLIM1,HLIM2U,HLIM2L,HLIM3H,HLIM3L,HLIM4,RSC
      REAL*8    X,SUM,VALUE,RFAC,AFGEN,KDIF,KDIR,COFAB

      CHARACTER CRPFIL*8,FILNAM*12,ENVSTR*50

      LOGICAL   EXISTS
C ----------------------------------------------------------------------
C --- compose filename
      CALL SHIFTR (CRPFIL)
      FILNAM = CRPFIL//'.CRP'
      CALL SHIFTL (FILNAM)
      CALL SHIFTR (ENVSTR)

C --- check if a file with simple crop data does exist
      INQUIRE (FILE=ENVSTR//FILNAM,EXIST=EXISTS)

      IF (.NOT.EXISTS) THEN
C ---   write error to log file
        WRITE (LOGF,92) ENVSTR//FILNAM
 92     FORMAT ('SWAP can not find the input file',/,A)
        STOP 'The .CRP file was not found: SWAP stopped !'
      ENDIF

C --- open file with crop data
      CALL FINDUNIT (10,CRP)
      OPEN (CRP,FILE=ENVSTR//FILNAM,STATUS='OLD',
     & access='sequential')

C --- phenology
      CALL RSINTR (CRP,FILNAM,LOGF,'IDEV',1,2,IDEV)
      IF (IDEV.EQ.1) then
        CALL RSINTR (CRP,FILNAM,LOGF,'LCC',1,366,LCC)
        CALL JMPLBL (CRP,FILNAM,LOGF,'KDIF')
      ELSEIF (IDEV.EQ.2) then
        CALL JMPLBL (CRP,FILNAM,LOGF,'TSUMEA')
        CALL RSDOUR (CRP,FILNAM,LOGF,'TSUMEA',0.0,10000.0,TSUMEA)
        CALL RSDOUR (CRP,FILNAM,LOGF,'TSUMAM',0.0,10000.0,TSUMAM)
        CALL RSDOUR (CRP,FILNAM,LOGF,'TBASE',-10.0, 20.0,TBASE)
      ENDIF

C --- assimilation                        
      CALL RSDOUR (CRP,FILNAM,LOGF,'KDIF',0.0,2.0,KDIF)
      CALL RSDOUR (CRP,FILNAM,LOGF,'KDIR',0.0,2.0,KDIR) 
     
C --- LAI or soil cover fraction 
      CALL RSINTR (CRP,FILNAM,LOGF,'SWGC',1,2,SWGC)
      IF (SWGC.EQ.1) then
        CALL RADOUR (CRP,FILNAM,LOGF,'GCTB',0.0,12.0,GCTB,72,IFND)
      ELSEIF (SWGC.EQ.2) then
        CALL RADOUR (CRP,FILNAM,LOGF,'GCTB',0.0,2.0,GCTB,72,IFND)
      ENDIF

C --- Crop factor or crop height
      CALL RSINTR (CRP,FILNAM,LOGF,'SWCF',1,2,SWCF)
      IF (SWCF.EQ.1) then
        CALL RADOUR (CRP,FILNAM,LOGF,'CFTB',0.0,2.01,CFTB,72,IFND)
      ELSEIF (SWCF.EQ.2) then
        CALL RADOUR (CRP,FILNAM,LOGF,'CFTB',0.0,1000.0,CFTB,72,IFND)
      ENDIF

C --- rooting depth
      CALL RADOUR (CRP,FILNAM,LOGF,'RDTB',0.0,1000.0,RDTB,72,IFND)

C --- yield response
      CALL RADOUR (CRP,FILNAM,LOGF,'KYTB',0.0,5.0,KYTB,72,IFND)

C --- water use
      CALL RSDOUR (CRP,FILNAM,LOGF,'HLIM1' ,-100.0,100.0,HLIM1)
      CALL RSDOUR (CRP,FILNAM,LOGF,'HLIM2U',-1000.0,100.0,HLIM2U)
      CALL RSDOUR (CRP,FILNAM,LOGF,'HLIM2L',-1000.0,100.0,HLIM2L)
      CALL RSDOUR (CRP,FILNAM,LOGF,'HLIM3H',-10000.0,100.0,HLIM3H)
      CALL RSDOUR (CRP,FILNAM,LOGF,'HLIM3L',-10000.0,100.0,HLIM3L)
      CALL RSDOUR (CRP,FILNAM,LOGF,'HLIM4' ,-16000.0,100.0,HLIM4)
      CALL RSDOUR (CRP,FILNAM,LOGF,'RSC',0.0,1000.0,RSC)
      CALL RSDOUR (CRP,FILNAM,LOGF,'ADCRH',0.0,5.0,ADCRH)
      CALL RSDOUR (CRP,FILNAM,LOGF,'ADCRL',0.0,5.0,ADCRL)
      CALL RSDOUR (CRP,FILNAM,LOGF,'ECMAX',0.0,20.0,ECMAX)
      CALL RSDOUR (CRP,FILNAM,LOGF,'ECSLOP',0.0,40.0,ECSLOP)
      CALL RSDOUR (CRP,FILNAM,LOGF,'COFAB',0.0,1.0,COFAB)

C --- read table with root distribution coefficients
      CALL RADOUR (CRP,FILNAM,LOGF,'RDCTB',0.0,100.0,RDCTB,22,IFND)

C --- normalize RDCTB
      SUM = 0.0
      DO 96 X = 0.05d0,1.0d0,0.1d0
        VALUE = AFGEN (RDCTB,22,X)
        SUM = SUM+0.1*VALUE
96    CONTINUE

      RFAC = 1/SUM
      DO 98 I = 1,IFND/2
98    RDCTB(I*2) = RDCTB(I*2)*RFAC

C --- close file with crop data
      CLOSE (CRP)

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE RDDRB (DRBFIL,ENVSTR,BAYRD,BAYRY,DRAMET,
     &IPOS,BASEGW,KHTOP,KHBOT,KVTOP,KVBOT,ZINTF,L,WETPER,ZBOTDR,ENTRES,
     &GEOFAC,NRLEVS,DRARES,INFRES,SWALLO,SWDTYP,OWLTAB,QDRTAB,LOGF,FMAY,
     &SWDIVD)
C ----------------------------------------------------------------------
C     Date               : 29/9/99                        
C     Purpose            : reading drainage parameters    
C                          for one agricultural year;  
C     Subroutines called : DATONR,SHIFTL,SHIFTR  
C     Functions called   : FROMTO                                      
C     File usage         : DRBFIL, file containing drainage data
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

      INTEGER   SWALLO(5),SWDTYP(5),FROMTO,BAYRD,BAYRY,FMAY
      INTEGER   DRAMET,DRB,NR,IPOS,NRLEVS,LOGF,SWDIVD

      REAL*8    L(5),ZBOTDR(5),OWLTAB(5,2*MAOWL),DRARES(5),INFRES(5)
      REAL*8    QDRTAB(50)
      REAL*8    BASEGW(2),WETPER(5),KHTOP,KHBOT,KVTOP,KVBOT
      REAL*8    ZINTF,ENTRES,GEOFAC

      CHARACTER DRBFIL*8,ENVSTR*50,FILNAM*12
C ----------------------------------------------------------------------
C --- local
      INTEGER  dd(MAOWL),mm(MAOWL),yy(MAOWL),IFND,I

      REAL*8   level(MAOWL),q(25),gwl(25)

      CHARACTER LINE*80

      LOGICAL COMMENT,BLANK,EXISTS
C ----------------------------------------------------------------------
C --- compose filename
      CALL SHIFTR (DRBFIL)
      FILNAM = DRBFIL//'.DRB'
      CALL SHIFTL (FILNAM)
      CALL SHIFTR (ENVSTR)

C --- check if a file with basic drainage data does exist
      INQUIRE (FILE=ENVSTR//FILNAM,EXIST=EXISTS)

      IF (.NOT.EXISTS) THEN
C ---   write error to log file
        WRITE (LOGF,92) ENVSTR//FILNAM
 92     FORMAT ('SWAP can not find the input file',/,A)
        STOP 'The .DRB file was not found: SWAP stopped !'
      ENDIF

C --- open file with drainage data
      CALL FINDUNIT (12,DRB)
      OPEN (DRB,FILE=ENVSTR//FILNAM,status='old',access='sequential')

C --- method to establish drainage/infiltration fluxes
      CALL RSINTR (DRB,FILNAM,LOGF,'DRAMET',1,3,DRAMET) 
      IF (dramet.ne.3) NRLEVS = 1

      IF (DRAMET.EQ.1) THEN
	  
	  CALL JMPLBL (DRB,FILNAM,LOGF,'LM1')
        CALL RSDOUR (DRB,FILNAM,LOGF,'LM1',1.0,1000.,L(1))
        L(1) = 100.0*L(1)
        IFND = 0
4       READ (DRB,'(A)') LINE
        CALL ANALIN (LINE,BLANK,COMMENT)
        IF (BLANK .OR. (COMMENT .AND. IFND .EQ. 0)) GOTO 4
        IF (COMMENT) GOTO 5
        IFND = IFND+1
        BACKSPACE (DRB)
        READ (DRB,*) gwl(IFND),q(IFND)
        GOTO 4
C ---   ready...
5       CONTINUE

        DO 7 I = 1,50
7       QDRTAB(I) = 0.0
        DO 9 I = 1,IFND
          QDRTAB(I*2-1) = ABS(gwl(I))
          QDRTAB(I*2)   = q(I)
9       CONTINUE

        CLOSE (DRB) 
        RETURN

      ELSEIF (DRAMET.EQ.2) THEN
   
        CALL JMPLBL (DRB,FILNAM,LOGF,'IPOS')

C ---   read profile characteristics
        CALL RSINTR (DRB,FILNAM,LOGF,'IPOS',1,5,IPOS)
        CALL RSDOUR (DRB,FILNAM,LOGF,'BASEGW',-1.E4,0.0,BASEGW(1))
        CALL RSDOUR (DRB,FILNAM,LOGF,'KHTOP',0.0,1000.0,KHTOP)

        IF (IPOS.GE.3) THEN
          CALL RSDOUR (DRB,FILNAM,LOGF,'KHBOT',0.0,1000.0,KHBOT)
        ELSE
          CALL JMPLBL (DRB,FILNAM,LOGF,'KVTOP')
        ENDIF
        IF (IPOS.GE.4) THEN
          CALL RSDOUR (DRB,FILNAM,LOGF,'KVTOP',0.0,1000.0,KVTOP)
          CALL RSDOUR (DRB,FILNAM,LOGF,'KVBOT',0.0,1000.0,KVBOT)  
        ELSE
          CALL JMPLBL (DRB,FILNAM,LOGF,'GEOFAC')
        ENDIF
        IF (IPOS.EQ.5) THEN
          CALL RSDOUR (DRB,FILNAM,LOGF,'GEOFAC',0.0,100.0,GEOFAC)
        ELSE
          CALL JMPLBL (DRB,FILNAM,LOGF,'ZINTF')
        ENDIF
        IF (IPOS.GE.3) THEN
          CALL RSDOUR (DRB,FILNAM,LOGF,'ZINTF',-1.E4,0.0,ZINTF)
        ELSE
          CALL JMPLBL (DRB,FILNAM,LOGF,'LM2')
        ENDIF

C ---   read drain characteristics
        CALL RSDOUR (DRB,FILNAM,LOGF,'LM2',1.0,1000.,L(1))
        L(1) = 100.0*L(1)
        CALL RSDOUR (DRB,FILNAM,LOGF,'WETPER',0.0,1000.0,WETPER(1))
        CALL RSDOUR (DRB,FILNAM,LOGF,'ZBOTDR',-1000.0,0.0,ZBOTDR(1))
        CALL RSDOUR (DRB,FILNAM,LOGF,'ENTRES',0.0,1000.0,ENTRES)

        CLOSE (DRB)
        RETURN

      ELSEIF (DRAMET.EQ.3) THEN
   
        CALL JMPLBL (DRB,FILNAM,LOGF,'NRLEVS')
        CALL RSINTR (DRB,FILNAM,LOGF,'NRLEVS',1,5,NRLEVS)

        IF (NRLEVS.GE.1) THEN
          CALL JMPLBL (DRB,FILNAM,LOGF,'DRARES1')
          CALL RSDOUR (DRB,FILNAM,LOGF,'DRARES1',10.0,1.0E5,DRARES(1))
          CALL RSDOUR (DRB,FILNAM,LOGF,'INFRES1',0.0,1.0E5,INFRES(1))
          CALL RSINTR (DRB,FILNAM,LOGF,'SWALLO1',1,3,SWALLO(1))
          IF (SWDIVD .EQ. 1) CALL RSDOUR (DRB,FILNAM,LOGF,'L1',1.0,
     &    1000.,L(1))
          L(1) = 100.0*L(1)
          CALL JMPLBL (DRB,FILNAM,LOGF,'ZBOTDR1')           
          CALL RSDOUR (DRB,FILNAM,LOGF,'ZBOTDR1',-1000.0,0.0,ZBOTDR(1))
          CALL RSINTR (DRB,FILNAM,LOGF,'SWDTYP1',1,2,SWDTYP(1))
          IF (SWDTYP(1).EQ.2) then
C ---       position file pointer
            READ (DRB,'(2X)')
            IFND = 0
14          READ (DRB,'(A)') LINE
            CALL ANALIN (LINE,BLANK,COMMENT)
            IF (BLANK .OR. (COMMENT .AND. IFND .EQ. 0)) GOTO 14
            IF (COMMENT) GOTO 15
            IFND = IFND+1
            BACKSPACE (DRB)
            READ (DRB,*) dd(IFND),mm(IFND),level(IFND)
            GOTO 14
C ---       ready...
15          CONTINUE
C ---       year is calculated
            DO 16 I = 1,IFND 
              if (mm(I).GE.FMAY) yy(I) = BAYRY
              if (mm(I).LT.FMAY) yy(I) = BAYRY+1
              CALL DATONR (yy(I),mm(I),dd(I),nr)
              OWLTAB(1,I*2)   = -ABS(LEVEL(I))
              OWLTAB(1,I*2-1) = FROMTO (BAYRD,BAYRY,NR,YY(I))+1
16          CONTINUE
          ENDIF
        ENDIF 

        IF (NRLEVS.GE.2) THEN
          CALL JMPLBL (DRB,FILNAM,LOGF,'DRARES2')
          CALL RSDOUR (DRB,FILNAM,LOGF,'DRARES2',10.0,1.0E5,DRARES(2))
          CALL RSDOUR (DRB,FILNAM,LOGF,'INFRES2',0.0,1.0E5,INFRES(2))
          CALL RSINTR (DRB,FILNAM,LOGF,'SWALLO2',1,3,SWALLO(2))
          IF (SWDIVD .EQ. 1) CALL RSDOUR (DRB,FILNAM,LOGF,'L2',1.0,
     &    1000.,L(2))
          L(2) = 100.0*L(2)
          CALL JMPLBL (DRB,FILNAM,LOGF,'ZBOTDR2')           
          CALL RSDOUR (DRB,FILNAM,LOGF,'ZBOTDR2',-1000.0,0.0,ZBOTDR(2))
          CALL RSINTR (DRB,FILNAM,LOGF,'SWDTYP2',1,2,SWDTYP(2))
          IF (SWDTYP(2).EQ.2) THEN
            READ (DRB,'(2X)')	      
            IFND = 0
24          READ (DRB,'(A)') LINE
            CALL ANALIN (LINE,BLANK,COMMENT)
            IF (BLANK .OR. (COMMENT .AND. IFND .EQ. 0)) GOTO 24
            IF (COMMENT) GOTO 25
            IFND = IFND+1
            BACKSPACE (DRB)
            READ (DRB,*) dd(IFND),mm(IFND),level(IFND)
            GOTO 24
C ---       ready...
25          CONTINUE
C ---       year is calculated
            DO 26 I = 1,IFND 
              if (mm(I).GE.FMAY) yy(I) = BAYRY
              if (mm(I).LT.FMAY) yy(I) = BAYRY+1
              CALL DATONR (yy(I),mm(I),dd(I),nr)
              OWLTAB(2,I*2)   = -ABS(LEVEL(I))
              OWLTAB(2,I*2-1) = FROMTO (BAYRD,BAYRY,NR,YY(I))+1
26          CONTINUE
          ENDIF
        ENDIF 

        IF (NRLEVS.GE.3) THEN
          CALL JMPLBL (DRB,FILNAM,LOGF,'DRARES3')
          CALL RSDOUR (DRB,FILNAM,LOGF,'DRARES3',10.0,1.0E5,DRARES(3))
          CALL RSDOUR (DRB,FILNAM,LOGF,'INFRES3',0.0,1.0E5,INFRES(3))
          CALL RSINTR (DRB,FILNAM,LOGF,'SWALLO3',1,3,SWALLO(3))
          IF (SWDIVD .EQ. 1) CALL RSDOUR (DRB,FILNAM,LOGF,'L3',1.0,
     &    1000.,L(3))
          L(3) = 100.0*L(3)
          CALL JMPLBL (DRB,FILNAM,LOGF,'ZBOTDR3')           
          CALL RSDOUR (DRB,FILNAM,LOGF,'ZBOTDR3',-1000.0,0.0,ZBOTDR(3))
          CALL RSINTR (DRB,FILNAM,LOGF,'SWDTYP3',1,2,SWDTYP(3))
          IF (SWDTYP(3).EQ.2) then
            READ (DRB,'(2X)')
            IFND = 0
34          READ (DRB,'(A)') LINE
            CALL ANALIN (LINE,BLANK,COMMENT)
            IF (BLANK .OR. (COMMENT .AND. IFND .EQ. 0)) GOTO 34
            IF (COMMENT) GOTO 35
            IFND = IFND+1
            BACKSPACE (DRB)
            READ (DRB,*) dd(IFND),mm(IFND),level(IFND)
            GOTO 34
C ---       ready...
35          CONTINUE
C ---       year is calculated
            DO 36 I = 1,IFND 
              if (mm(I).GE.FMAY) yy(I) = BAYRY
              if (mm(I).LT.FMAY) yy(I) = BAYRY+1
              CALL DATONR (yy(I),mm(I),dd(I),nr)
              OWLTAB(3,I*2)   = -ABS(LEVEL(I))
              OWLTAB(3,I*2-1) = FROMTO (BAYRD,BAYRY,NR,YY(I))+1
36          CONTINUE
          ENDIF
        ENDIF 

        IF (NRLEVS.GE.4) THEN
          CALL JMPLBL (DRB,FILNAM,LOGF,'DRARES4')
          CALL RSDOUR (DRB,FILNAM,LOGF,'DRARES4',10.0,1.0E5,DRARES(4))
          CALL RSDOUR (DRB,FILNAM,LOGF,'INFRES4',0.0,1.0E5,INFRES(4))
          CALL RSINTR (DRB,FILNAM,LOGF,'SWALLO4',1,3,SWALLO(4))
          IF (SWDIVD .EQ. 1) CALL RSDOUR (DRB,FILNAM,LOGF,'L4',1.0,
     &    1000.,L(4))
          L(4) = 100.0*L(4)
          CALL JMPLBL (DRB,FILNAM,LOGF,'ZBOTDR4')           
          CALL RSDOUR (DRB,FILNAM,LOGF,'ZBOTDR4',-1000.0,0.0,ZBOTDR(4))
          CALL RSINTR (DRB,FILNAM,LOGF,'SWDTYP4',1,2,SWDTYP(4))
          IF (SWDTYP(4).EQ.2) then
            READ (DRB,'(2X)')
            IFND = 0
44          READ (DRB,'(A)') LINE
            CALL ANALIN (LINE,BLANK,COMMENT)
            IF (BLANK .OR. (COMMENT .AND. IFND .EQ. 0)) GOTO 44
            IF (COMMENT) GOTO 45
            IFND = IFND+1
            BACKSPACE (DRB)
            READ (DRB,*) dd(IFND),mm(IFND),level(IFND)
            GOTO 44
C ---       ready...
45          CONTINUE
C ---       year is calculated
            DO 46 I = 1,IFND 
              if (mm(I).GE.FMAY) yy(I) = BAYRY
              if (mm(I).LT.FMAY) yy(I) = BAYRY+1
              CALL DATONR (yy(I),mm(I),dd(I),nr)
              OWLTAB(4,I*2)   = -ABS(LEVEL(I))
              OWLTAB(4,I*2-1) = FROMTO (BAYRD,BAYRY,NR,YY(I))+1
46          CONTINUE
          ENDIF
        ENDIF 

        IF (NRLEVS.GE.5) THEN
          CALL JMPLBL (DRB,FILNAM,LOGF,'DRARES5')
          CALL RSDOUR (DRB,FILNAM,LOGF,'DRARES5',10.0,1.0E5,DRARES(5))
          CALL RSDOUR (DRB,FILNAM,LOGF,'INFRES5',0.0,1.0E5,INFRES(5))
          CALL RSINTR (DRB,FILNAM,LOGF,'SWALLO5',1,3,SWALLO(5))
          IF (SWDIVD .EQ. 1) CALL RSDOUR (DRB,FILNAM,LOGF,'L5',1.0,
     &    1000.,L(5))
          L(5) = 100.0*L(5)
          CALL JMPLBL (DRB,FILNAM,LOGF,'ZBOTDR5')           
          CALL RSDOUR (DRB,FILNAM,LOGF,'ZBOTDR5',-1000.0,0.0,ZBOTDR(5))
          CALL RSINTR (DRB,FILNAM,LOGF,'SWDTYP5',1,2,SWDTYP(5))
          IF (SWDTYP(5).EQ.2) then
            READ (DRB,'(2X)')
            IFND = 0
54          READ (DRB,'(A)') LINE
            CALL ANALIN (LINE,BLANK,COMMENT)
            IF (BLANK .OR. (COMMENT .AND. IFND .EQ. 0)) GOTO 54
            IF (COMMENT) GOTO 55
            IFND = IFND+1
            BACKSPACE (DRB)
            READ (DRB,*) dd(IFND),mm(IFND),level(IFND)
            GOTO 54
C ---       ready...
55          CONTINUE
C ---       year is calculated
            DO 56 I = 1,IFND 
              if (mm(I).GE.FMAY) yy(I) = BAYRY
              if (mm(I).LT.FMAY) yy(I) = BAYRY+1
              CALL DATONR (yy(I),mm(I),dd(I),nr)
              OWLTAB(5,I*2)   = -ABS(LEVEL(I))
              OWLTAB(5,I*2-1) = FROMTO (BAYRD,BAYRY,NR,YY(I))+1
56          CONTINUE
          ENDIF
        ENDIF 
      ENDIF 

C --- close file with drainage data
      CLOSE (DRB)         

      RETURN
      END


C ----------------------------------------------------------------------
      SUBROUTINE RDDRE (SRFFIL,ENVSTR,RSRO,NRSRF,NRPRI,NRSEC,L,
     & ZBOTDR,WIDTHR,TALUDR,RDRAIN,RINFI,RENTRY,REXIT,GWLINF,SWDTYP,
     & WLPTAB,BAYRD,BAYRY,SWSEC,WLS,OSSWLM,NMPER,WLSTAR,IMPEND,SWMAN,
     & WSCAP,SWQHR,HBWEIR,ALPHAW,BETAW,NQH,HQHTAB,QQHTAB,DROPR,WLSMAN,
     & GWLCRIT,WLSTAB,STTAB,WSTINI,WST,T,WLSBAK,SWSRF,NPHASE,HCRIT,
     & HDEPTH,VCRIT,WLP,NODHD,NUMNOD,DZ,WLDIP,NUMADJ,LOGF,FMAY,INTWL)
C ----------------------------------------------------------------------
C     Date               : 29/9/99                    
C     Purpose            : reading multilevel drainage characteristics 
C                          and specification of the surface water system
C                          for a period up to one year;  

C --- 1 Reading .DRE input file
C --- 2 Initializations

C --- Initializations:
C -1- wlp: water level in primary system   (SWSRF = 3)
C -2- wls: water level in secondary system (SWSRF = 2 or 3, SWSEC = 1 or 2) 
C -3- HBWEIR(IMPER) in case of table discharge relation (SWQHR = 2)
C -4- NUMADJ: number of target level adjustments
C -5- sttab(22,2) table with storage as a function of water level
C ---    sttab(i,1) contains levels:
C ---    i=1: +100 cm; i=2: 0 cm; i=22: bottom of deepest dr. medium
C ---    sttab(i,2) contains storage expressed as surface layer 

C     Subroutines called :  
C     Functions called   :                               
C     File usage         : DREFIL, file containing drainage data
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      INTEGER   NRPRI,NRSEC,NRSRF,SWDTYP(5),NMPER,SWMAN(10),IMPEND(10)
      INTEGER   SWQHR,NQH(10),BAYRD,BAYRY,SWSEC,SWSRF,NPHASE(10)
      INTEGER   NODHD(10),NUMNOD,NUMADJ,LOGF,FMAY,INTWL(10)

      REAL*8    L(5),ZBOTDR(5),WIDTHR(5),TALUDR(5),RDRAIN(5)
      REAL*8    RINFI(5),RENTRY(5),REXIT(5),GWLINF(5),WLPTAB(2*MAWLP)
      REAL*8    RSRO
      REAL*8    WLS,OSSWLM,WLSTAR,WSCAP(10),HBWEIR(10),ALPHAW(10)
      REAL*8    BETAW(10),HQHTAB(10,10),QQHTAB(10,10),DROPR(10)
      REAL*8    WLSMAN(10,10),WLSTAB(2*MAWLS),STTAB(22,2),T,HDEPTH(10)
      REAL*8    WSTINI,WST,WLSBAK(4),HCRIT(10,10),VCRIT(10,10)
      REAL*8    AFGEN,WLP,DZ(MACP),GWLCRIT(10,10),WLDIP(10)

      CHARACTER SRFFIL*8,ENVSTR*50

C ----------------------------------------------------------------------
C --- local
      INTEGER   DRE,level,D,M,Y,NR,FROMTO,ITAB,I
      INTEGER   iph,NRMAN1,NRMAN2,NODE,IMPER,IMPERB,IMPERI,IFND
   
      REAL*8    VALUE,wdepth,wbreadth,wvolum,DEP,WSTLEV,SOFCU,ALTCU

      LOGICAL   FLWEIR(10),EXISTS(10),FLZERO(10),BLANK,COMMENT

      CHARACTER FILNAM*12,LINE*80

C ----------------------------------------------------------------------
C --- compose filename
      CALL SHIFTR (SRFFIL)
      FILNAM = SRFFIL//'.DRE'
      CALL SHIFTL (FILNAM)
      CALL SHIFTR (ENVSTR)

C --- open file with drainage data
      CALL Findunit (10,DRE)
      OPEN (DRE,FILE = ENVSTR//FILNAM,STATUS='OLD',
     $ACCESS = 'SEQUENTIAL')

C --- altitude of control unit (relative to reference level)
      CALL RSDOUR (DRE,FILNAM,LOGF,'ALTCU',-300000.0,300000.0,ALTCU)

C --- section 1

C --- drainage resistance of surface runoff
      CALL RSDOUR (DRE,FILNAM,LOGF,'RSRO',0.001,1.0,RSRO)

C --- number of drainage levels
      CALL RSINTR (DRE,FILNAM,LOGF,'NRSRF',1,5,NRSRF)

C --- characteristics of each drainage level 
      IFND = 0
4     READ (DRE,'(A)') LINE
      CALL ANALIN (LINE,BLANK,COMMENT)
      IF (BLANK .OR. (COMMENT .AND. IFND .EQ. 0)) GOTO 4
      IF (COMMENT) GOTO 6 
        IFND = IFND+1
        BACKSPACE (DRE)
        READ (DRE,*) level, swdtyp(level)
        BackSpace (DRE)
        If (swdtyp(level) .EQ.0 ) then
C ---     open      
          READ (DRE,*) level,swdtyp(level),l(level),zbotdr(level),
     &      gwlinf(level),rdrain(level),rinfi(level),rentry(level),
     &      rexit(level),widthr(level),taludr(level)
        else
C ---     closed 
          READ (DRE,*) level,swdtyp(level),l(level),zbotdr(level),
     &      gwlinf(level),rdrain(level),rinfi(level)

        endif
        if (level.ne.IFND) stop 'RDDRE: dr. level index not consistent' 
C ---   necessairy conversions
        l(level)      = l(level)*100.0
        ZBOTDR(level) = ZBOTDR(level) -ALTCU
C ---   some security checks....
C ---   1 - levels must be ordered, starting with the deepest
C ---   2 - zbotdr always below surface level
C ---   3 - gwlinf must be below bottom of deepest drainage medium
        if (level .gt. 1) then
          if (zbotdr(level) .lt. zbotdr(level-1))
     &    stop 'RDDRE: levels must be ordered, starting with deepest'
        endif
        if (zbotdr(level).gt.0.0)
     &    stop 'RDDRE: zbotdr must be negative'
        if (gwlinf(level).gt.zbotdr(level))
     &    stop 'RDDRE: gwlinf higher than zbotdr'

C ---   next record...
        GOTO 4
6     CONTINUE
      IF (IFND.NE. NRSRF)
     &  stop 'RDDRE: incorrect number of drainage levels described'

C --- section 2a

      CALL RSINTR (DRE,FILNAM,LOGF,'SWSRF',1,3,SWSRF)
      IF (SWSRF.EQ.3) THEN
        NRPRI = 1
        NRSEC = NRSRF-NRPRI
      ELSEIF (SWSRF.EQ.2) THEN
        NRPRI = 0
        NRSEC = NRSRF-NRPRI
      ELSEIF (SWSRF.EQ.1) THEN
C ---   no surface water system...ready
        Close (DRE)
        Return
      ENDIF
C --- is dit wel goed ?
      IF (NRSEC.LT.1) STOP 'RDDRE - secondary system not specified'

      If (SWDTYP(1+NRPRI).NE.0)
     &  STOP 'RDDRE - Deepest sec. level must be open'

C --- section 2b

C --- read table with water levels in the primary system (SWSRF=3)
      IF (SWSRF.EQ.3) THEN
C ---   init table
        DO 8 I = 1,2*MAWLP
8       WLPTAB(I) = 0.0
C ---   position file pointer
        IFND = 0
12      READ (DRE,'(A)') LINE
        CALL ANALIN (LINE,BLANK,COMMENT)
        IF (BLANK .OR. (COMMENT .AND. IFND .EQ. 0)) GOTO 12
        IF (COMMENT) GOTO 14 
          IFND = IFND+1
          BACKSPACE (DRE)
          READ (DRE,*) D,M,VALUE
C ---     calculate year 
          IF (M.GE.FMAY) Y = BAYRY
          IF (M.LT.FMAY) Y = BAYRY+1
          CALL DATONR (Y,M,D,NR)
          IF (IFND.GT.MAWLP) STOP 'RDDRE - WLPTAB too large'
          WLPTAB(IFND*2) = VALUE -ALTCU
          if (VALUE.gt.0.0) 
     &      STOP 'RDDRE: water level in primary system must be negative'
          WLPTAB(IFND*2-1) = FROMTO (BAYRD,BAYRY,NR,Y)+1
C ---     next record...
          GOTO 12
14      CONTINUE
C ---   set initial value
        wlp = AFGEN (WLPTAB,2*MAWLP,T+1.0d0)

C ---   Ready in case of only a primary system
C       if (NRSEC.LT.1) then
C          CLOSE(DRE)
C          Return
C       endif 

      ELSE
        CALL JMPLBL (DRE,FILNAM,LOGF,'SWSEC')
      ENDIF

C --- Section 2c

      CALL RSINTR (DRE,FILNAM,LOGF,'SWSEC',1,2,SWSEC)

C --- Section 3

      IF (SWSEC.EQ.1) THEN
C ---   surface water level of secondary system is input
C ---   position file pointer
510     READ (DRE,'(A)') LINE
        CALL ANALIN (LINE,BLANK,COMMENT)
        IF (BLANK.OR.COMMENT) GOTO 510  
        BACKSPACE (DRE)

C ---   init table
        DO 60 I = 1,2*MAWLS
60      WLSTAB(I) = 0.0

        IFND = 0
512     READ (DRE,'(A)') LINE
        CALL ANALIN (LINE,BLANK,COMMENT)
        IF (BLANK) GOTO 512
        IF (COMMENT) GOTO 514 

          IFND = IFND+1
          BACKSPACE (DRE)
          READ (DRE,*) D,M,VALUE
C ---     calculate year 
          IF (M.GE.FMAY) Y = BAYRY
          IF (M.LT.FMAY) Y = BAYRY+1
          CALL DATONR (Y,M,D,NR)
          IF (IFND.GT.MAWLS) STOP 'RDDRE - WLSTAB too large'
          WLSTAB(IFND*2)   = VALUE - ALTCU
          WLSTAB(IFND*2-1) = FROMTO (BAYRD,BAYRY,NR,Y)+1
          GOTO 512
514     CONTINUE
C ---   set initial value
        wls = AFGEN (WLSTAB,2*MAWLS,T+1.0d0)

C ---   Section 4a

C --- surface water level of secondary system is simulated
      ELSEIF (SWSEC.EQ.2) THEN
        CALL JMPLBL (DRE,FILNAM,LOGF,'WLACT')
        CALL RSDOUR (DRE,FILNAM,LOGF,'WLACT',real(zbotdr(1+NRPRI)+ALTCU)
     &       ,real(ALTCU),wls)
        WLS = WLS - ALTCU
        CALL RSDOUR (DRE,FILNAM,LOGF,'OSSWLM',0.0,10.0,osswlm)

C ---   Section 4b

        CALL RSINTR (DRE,FILNAM,LOGF,'NMPER',1,10,nmper)
        wlstar = wls
        NRMAN1 = 0
        NRMAN2 = 0

C ---   position file pointer
16      READ (DRE,'(A)') LINE
        CALL ANALIN (LINE,BLANK,COMMENT)
        IF (BLANK.OR.COMMENT) GOTO 16 
        BACKSPACE (DRE)

18      READ (DRE,'(A)') LINE
        CALL ANALIN (LINE,BLANK,COMMENT)
        IF (BLANK) GOTO 18
        IF (COMMENT) GOTO 20 

          BACKSPACE (DRE)
          READ (DRE,*) imper,D,M,swman(imper),wscap(imper),wldip(imper),
     &       intwl(imper)
          if (swman(imper).eq.2 .and. intwl(imper).lt.1) then
            STOP 'RDDRE: INTWL (management interval) must be >= 1 day'
          endif
          wldip(imper) = abs(wldip(imper))

C ---     for each type of management: count number of periods 
          if (swman(imper).eq.1) then 
            NRMAN1 = NRMAN1+1
          elseif (swman(imper).eq.2) then
            NRMAN2 = NRMAN2+1
          else
            STOP 'RDDRE: SWMAN out of range'
          endif

C ---     calculate year 
          IF (M.GE.FMAY) Y = BAYRY
          IF (M.LT.FMAY) Y = BAYRY+1
          CALL DATONR (Y,M,D,NR)
          impend(imper) = FROMTO (BAYRD,BAYRY,NR,Y)+1

C ---   next record...
        GOTO 18
20      IF ((NRMAN1+NRMAN2).NE.NMPER)
     &  STOP 'RDDRE: NRMAN1+NRMAN2 does not match NMPER'

C ---   type of discharge relationship
        CALL RSINTR (DRE,FILNAM,LOGF,'SWQHR',1,2,SWQHR)

        IF (SWQHR.EQ.1) THEN

C ---     Section 4c

          CALL RSDOUR (DRE,FILNAM,LOGF,'SOFCU',0.1 ,100000.0,SOFCU)

C ---     position file pointer
22        READ (DRE,'(A)') LINE
          CALL ANALIN (LINE,BLANK,COMMENT)
          IF (BLANK.OR.COMMENT) GOTO 22 
          BACKSPACE (DRE)

          IMPERB = 0
          IMPERI = 0

24        READ (DRE,'(A)') LINE
          CALL ANALIN (LINE,BLANK,COMMENT)
          IF (BLANK) GOTO 24
          IF (COMMENT) GOTO 26 

            BACKSPACE (DRE)
            READ (DRE,*) imper,hbweir(imper),alphaw(imper),betaw(imper)
            hbweir(imper)= hbweir(imper)-ALTCU
C ---       correction for units
            alphaw(imper) = alphaw(imper)*(8.64*100**
     &                      (1.0d0-betaw(imper))/SOFCU)     
            if (hbweir(imper).lt.zbotdr(1+NRPRI)) then
              write(6,*) 'RDDRE - weir crest level below bottom of'
              write(6,*) 'deepest channel of secondary system'
              stop
            endif
c --- check for target level above channel bottom when supply is 
c --- attempted (system may never become dry in this case)
            if (swman(imper).eq.1 .and. wscap(imper).gt.1.d-7 .and.
     &         (hbweir(imper)-wldip(imper)).lt.
     &         (zbotdr(1+NRPRI)+1.d-4)) then
              STOP 'RDDRE: HBWEIR/WLDIP supply not possible =< zbotdr !'
            endif

            IF (imper.ne.imperb) then
              imperb = imper
              imperi = imperi + 1
            ELSE
              STOP 'RDDRE: 4c IMPER not unique'
            ENDIF
C ---     next record
          GOTO 24

C ---     check number of records.. 
26        IF (IMPERI.NE.NMPER)
     &    STOP 'INISRF: section 4c - not enough records'

        ELSEIF (SWQHR.EQ.2) THEN

C ---     Section 4d

          CALL JMPLBL (DRE,FILNAM,LOGF,'LABEL4d')
          CALL RSINTR (DRE,FILNAM,LOGF,'LABEL4d',0,10,I)

C ---     position file pointer
28        READ (DRE,'(A)') LINE
          CALL ANALIN (LINE,BLANK,COMMENT)
          IF (BLANK.OR.COMMENT) GOTO 28 
          BACKSPACE (DRE)

          IMPERB = 0
          IMPERI = 0

          DO 34 IMPER = 1,10
            NQH (IMPER)    = 0
            FLWEIR (IMPER) = .FALSE.
            EXISTS (IMPER) = .FALSE.
            FLZERO (IMPER) = .FALSE.
34        CONTINUE

30        READ (DRE,'(A)') LINE
          CALL ANALIN (LINE,BLANK,COMMENT)
          IF (BLANK) GOTO 30
          IF (COMMENT) GOTO 32 

            BACKSPACE (DRE)
            READ (DRE,*) IMPER,ITAB,HQHTAB(IMPER,ITAB),
     &        QQHTAB(IMPER,ITAB)
            HQHTAB(IMPER,ITAB)=HQHTAB(IMPER,ITAB)-AltCu
            NQH(IMPER) = NQH(IMPER)+1
            EXISTS(IMPER) = .TRUE.
            IF (QQHTAB(IMPER,ITAB).LT.0.000001d0) FLZERO(IMPER)=.TRUE.
            if (hqhtab(imper,itab).lt.zbotdr(1+NRPRI)) then
              write(6,*) 'RDDRE - level in q-h table below bottom of '
              write(6,*) 'deepest channel of secondary system'
              stop
            endif
            IF (imper.ne.imperb) then
              imperb = imper
              imperi = imperi + 1
            ENDIF

C ---       establish HBWEIR (at level where QTAB = 0) 
            if (QQHTAB(IMPER,ITAB).LT.0.0001d0.AND..NOT.FLWEIR(IMPER)) 
     &      THEN
              HBWEIR(IMPER) = HQHTAB(IMPER,ITAB)
              FLWEIR(IMPER) = .TRUE.
            ENDIF 

C ---       consistency checks
            if (NQH(IMPER).NE.ITAB)
     &        STOP 'RDDRE: QH-table / IMPER - ITAB mismatch'
            if (ITAB.EQ.1) THEN
            if (ABS(HQHTAB(IMPER,ITAB)-100.0d0).GT.0.00001d0)
     &        STOP 'RDDRE: first value in HTAB should be AltCu+100.0'
            endif
            if (ITAB.GT.1) THEN
              if ((HQHTAB(IMPER,ITAB).GE.HQHTAB(IMPER,ITAB-1)).OR.
     &           (QQHTAB(IMPER,ITAB).GT.QQHTAB(IMPER,ITAB-1)))
     &          STOP 'RDDRE: QH-table - no descending values'
            endif

C ---     next record
          GOTO 30

C ---     check number of periods.. 
32        IF (IMPERI.NE.NMPER)
     &    STOP 'RDDRE: section 4d - number of periods incorrect'

C ---     check that QQHTAB goes down to zero
          DO 36 imper = 1,10
            IF (EXISTS(IMPER).AND..NOT.FLZERO(IMPER))
     &      STOP 'RDDRE: QQHTAB not going down to zero'
36         CONTINUE

        ENDIF

C ---   position filepointer at 4e
        CALL JMPLBL (DRE,FILNAM,LOGF,'LABEL4e')
        CALL RSINTR (DRE,FILNAM,LOGF,'LABEL4e',0,10,I)

        IF (NRMAN2.GT.0) THEN

38        READ (DRE,'(A)') LINE
          CALL ANALIN (LINE,BLANK,COMMENT)
          IF (BLANK.OR.COMMENT) GOTO 38 
          BACKSPACE (DRE)

          IMPERB = 0
          IMPERI = 0

C ---     read table with drop rates 
40        READ (DRE,'(A)') LINE
          CALL ANALIN (LINE,BLANK,COMMENT)
          IF (BLANK) GOTO 40
          IF (COMMENT) GOTO 42 

            BACKSPACE (DRE)
            READ (DRE,*) imper,dropr(imper),hdepth(imper)
            hdepth(imper) = -abs(hdepth(imper))
C ---       determine compartment number related to hdepth
            DEP = 0.0
            DO 44 NODE = 1,NUMNOD
              NODHD(imper) = NODE
              DEP = DEP-DZ(NODE)
              IF (hdepth(imper).ge.DEP-1.0d-6) GOTO 46
44          CONTINUE
46          CONTINUE

            IF (SWMAN(imper) .NE. 2)
     &        STOP 'RDDRE: #4e swman - imper mismatch'
            IF (imper.ne.imperb) then
              imperb = imper
              imperi = imperi + 1
            ELSE
              STOP 'RDDRE: two drop rates same period'
            ENDIF
C ---     next record
          GOTO 40
42        IF (IMPERI .NE. NRMAN2)
     &    STOP 'RDDRE: #4e number of periods for drop rate incorrect'
 
48        READ (DRE,'(A)') LINE
          CALL ANALIN (LINE,BLANK,COMMENT)
          IF (BLANK.OR.COMMENT) GOTO 48 
          BACKSPACE (DRE)

          IMPERB = 0
          IMPERI = 0

          DO 54 IMPER = 1,10
54        NPHASE(IMPER) = 0

          IFND = 0
50        READ (DRE,'(A)') LINE
          CALL ANALIN (LINE,BLANK,COMMENT)
          IF (BLANK) GOTO 50
          IF (COMMENT) GOTO 52 

            BACKSPACE (DRE)
            READ (DRE,*) imper,iph,wlsman(imper,iph),
     &      gwlcrit(imper,iph),HCRIT(imper,iph),VCRIT(imper,iph)
            NPHASE(IMPER) = NPHASE(IMPER) + 1
            wlsman(imper,iph) = wlsman(imper,iph)-ALTCU
            if (wlsman(imper,iph).lt.zbotdr(1+NRPRI)) then
              write(6,*) 'RDDRE - level of automatic weir below bottom'
              write(6,*) 'of deepest channel of secondary system'
              stop
            endif
            IF (SWMAN(imper) .NE. 2)
     &      STOP 'RDDRE: #4e swman - imper mismatch'
            IF (imper.ne.imperb) then
              imperb = imper
              imperi = imperi + 1
            ENDIF

C ---     next record...
          GOTO 50
52        IF (IMPERI .NE. NRMAN2)
     &    STOP 'RDDRE: #4e number of periods incorrect'

C ---     consistency checks WLSMAN, GWLCRIT, HCRIT and VCRIT
          DO 446 imper = 1,nmper
            IF (SWMAN(imper).EQ.2) THEN
              if (abs(gwlcrit(imper,1)) .gt. 0.01d0) 
     &          STOP 'RDDRE: #4e - GWLCRIT(1) must be 0.'
              if (abs(vcrit(imper,1)) .gt. 0.01d0) 
     &          STOP 'RDDRE: #4e - VCRIT(1) must be 0.'
              if (abs(hcrit(imper,1)) .gt. 0.01d0) 
     &          STOP 'RDDRE: #4e - HCRIT(1) must be 0.'

              if (swman(imper).eq.2 .and. 
     &          hbweir(imper).gt. (wlsman(imper,1)-0.99999d0)) then
               STOP 'RDDRE: #4e - HBWEIR within 1 cm of wlsman(1)' 
              endif

              DO 448 iph = 2,nphase(imper)
                if (wlsman(imper,iph).LT.wlsman(imper,iph-1))
     &          STOP 'RDDRE: #4e - WLSMAN inconsistent'
                if (GWLCRIT(imper,iph).GT.GWLCRIT(imper,iph-1))
     &          STOP 'RDDRE: #4e - GWLCRIT inconsistent'
                if (HCRIT(imper,iph).GT.HCRIT(imper,iph-1))
     &          STOP 'RDDRE: #4e - HCRIT inconsistent'
                if (VCRIT(imper,iph).LT.VCRIT(imper,iph-1))
     &          STOP 'RDDRE: #4e - VCRIT inconsistent'
448           CONTINUE
            ENDIF 
446       CONTINUE
        ENDIF
      ENDIF
C ----------------------------------------------------------------------
C --- initialize counter for number of target level adjustments
      numadj = 0

C --- sttab(i,1) contains depths

C --- sw-levels, to be used in piece-wise linear functions (first level)
C --- is 100 cm above soil surface to allow for situations with ponding)
      sttab(1,1) = 100.0d0
      sttab(2,1) =   0.0
      do 100 i = 3,22
C ---   layer between surface level and deepest
C ---   drain/channel bottom is divided into 20 compartments
        sttab(i,1) = zbotdr(1+nrpri)*(i-2)/20.0
100   continue

C --- sttab(i,2) contains storage expressed in cm

C --- calculation of surface water storage (in cm, i.e. volume
C --- per unit area), as a function of sw-level, only for open channels:
      do 104 i = 1,22
        sttab(i,2) = 0.0
        do 108 level = 1+nrpri,nrsrf
          if (swdtyp(level).eq.0.and.sttab(i,1).gt.zbotdr(level)) then
C ---       for levels above soil surface the volume-increment is
C ---       computed for a rectangle, and not a trapezium
            if (sttab(i,1) .le. 0.0) then
              wdepth   = sttab(i,1)-zbotdr(level)
              wvolum   = wdepth*(widthr(level)+wdepth/taludr(level))
            else
              wdepth   = -zbotdr(level)
              wvolum   = wdepth*(widthr(level)+wdepth/taludr(level))
              wbreadth = widthr(level) + 2*wdepth/taludr(level)
              wdepth   = sttab(i,1)
              wvolum   = wvolum + wbreadth*wdepth
            endif
            sttab(i,2) = sttab(i,2)+wvolum/l(level)
          endif
108     continue
104   continue

C --- initial storage wstini
      wstini = wstlev (wls,sttab)
      wst = wstini
C --- initialize memorization of wls for most recent 4 timesteps 
      do 200 i=1,4
        wlsbak(i) = 0.0
 200  continue

C --- close input file with lateral boundary conditions
      CLOSE (DRE)         

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE RDGRASS (CRPFIL,ENVSTR,TDWI,LAIEM,RGRLAI,SLATB,
     &  SSA,SPAN,TBASE,KDIF,KDIR,EFF,AMAXTB,TMPFTB,TMNFTB,CVL,CVR,CVS,
     &  Q10,RML,RMR,RMS,RFSETB,FRTB,FLTB,FSTB,PERDL,RDRRTB,
     &  RDRSTB,HLIM1,HLIM2U,HLIM2L,HLIM3H,HLIM3L,HLIM4,RSC,ADCRH,
     &  ADCRL,ECMAX,ECSLOP,COFAB,RDI,RRI,RDC,RDCTB,LOGF)
C ----------------------------------------------------------------------
C     Date               : 14/1/99   
C     Purpose            : get crop parameters for grass     
C     Subroutines called : SHIFTR, SHIFTL
C     Functions called   : AFGEN
C     File usage         : CRPFIL, file containing crop parameters
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER CRP,I,LOGF,IFND
 
      REAL*8 SLATB(30),AMAXTB(30),TMPFTB(30)
      REAL*8 TMNFTB(30),RFSETB(30),FRTB(30),FLTB(30),FSTB(30),RDRRTB(30)
      REAL*8 RDRSTB(30),KDIF,KDIR,LAIEM,ECMAX,ECSLOP,COFAB,RSC
      REAL*8 HLIM1,HLIM2U,HLIM2L,HLIM3H,HLIM3L,HLIM4,RDCTB(22)
      REAL*8 X,SUM,RFAC,VALUE,ADCRH,ADCRL,AFGEN,CVL,CVR,CVS,EFF,RMR
      REAL*8 PERDL,Q10,RDC,RDI,RGRLAI,RML,RMS,RRI,SPAN,SSA,TBASE,TDWI

      CHARACTER CRPFIL*8,ENVSTR*50,FILNAM*12

      LOGICAL   EXISTS

C ----------------------------------------------------------------------
C --- compose filename 
      CALL SHIFTR (CRPFIL)
      FILNAM = CRPFIL//'.CRP'
      CALL SHIFTL (FILNAM)
      CALL SHIFTR (ENVSTR)

C --- check if a file with detailed grass data does exist
      INQUIRE (FILE=ENVSTR//FILNAM,EXIST=EXISTS)

      IF (.NOT.EXISTS) THEN
C ---   write error to log file
        WRITE (LOGF,92) ENVSTR//FILNAM
 92     FORMAT ('SWAP can not find the input file',/,A)
        STOP 'The .CRP file was not found: SWAP stopped !'
      ENDIF

C --- open file with crop data
      Call Findunit (10,CRP) 
      OPEN (CRP,FILE=ENVSTR//FILNAM,STATUS='OLD',
     & access='sequential')

C --- initial
      CALL RSDOUR (CRP,FILNAM,LOGF,'TDWI',0.0,10000.0,TDWI)
      CALL RSDOUR (CRP,FILNAM,LOGF,'LAIEM',0.0,10.0,LAIEM)
      CALL RSDOUR (CRP,FILNAM,LOGF,'RGRLAI',0.0,1.0,RGRLAI)

C --- green area
      CALL RADOUR (CRP,FILNAM,LOGF,'SLATB',0.0,366.0,SLATB,30,IFND)
      CALL RSDOUR (CRP,FILNAM,LOGF,'SSA',0.0,1.0,SSA)
      CALL RSDOUR (CRP,FILNAM,LOGF,'SPAN',0.0,366.0,SPAN)
      CALL RSDOUR (CRP,FILNAM,LOGF,'TBASE',-10.0,30.0,TBASE)

C --- assimilation
      CALL RSDOUR (CRP,FILNAM,LOGF,'KDIF',0.0,2.0,KDIF)
      CALL RSDOUR (CRP,FILNAM,LOGF,'KDIR',0.0,2.0,KDIR)
      CALL RSDOUR (CRP,FILNAM,LOGF,'EFF',0.0,10.0,EFF)
      CALL RADOUR (CRP,FILNAM,LOGF,'AMAXTB',0.0,366.0,AMAXTB,30,IFND)
      CALL RADOUR (CRP,FILNAM,LOGF,'TMPFTB',-10.0,50.0,TMPFTB,30,IFND)
      CALL RADOUR (CRP,FILNAM,LOGF,'TMNFTB',-10.0,50.0,TMNFTB,30,IFND)

C --- conversion of assimilates into biomass
      CALL RSDOUR (CRP,FILNAM,LOGF,'CVL',0.0,1.0,CVL)
      CALL RSDOUR (CRP,FILNAM,LOGF,'CVR',0.0,1.0,CVR)
      CALL RSDOUR (CRP,FILNAM,LOGF,'CVS',0.0,1.0,CVS)

C --- maintenance respiration
      CALL RSDOUR (CRP,FILNAM,LOGF,'Q10',0.0,5.0,Q10)
      CALL RSDOUR (CRP,FILNAM,LOGF,'RML',0.0,1.0,RML)
      CALL RSDOUR (CRP,FILNAM,LOGF,'RMR',0.0,1.0,RMR)
      CALL RSDOUR (CRP,FILNAM,LOGF,'RMS',0.0,1.0,RMS)
      CALL RADOUR (CRP,FILNAM,LOGF,'RFSETB',0.0,366.0,RFSETB,30,IFND)

C --- partitioning
      CALL RADOUR (CRP,FILNAM,LOGF,'FRTB',0.0,366.0,FRTB,30,IFND)
      CALL RADOUR (CRP,FILNAM,LOGF,'FLTB',0.0,366.0,FLTB,30,IFND)
      CALL RADOUR (CRP,FILNAM,LOGF,'FSTB',0.0,366.0,FSTB,30,IFND)

C --- death rates
      CALL RSDOUR (CRP,FILNAM,LOGF,'PERDL',0.0,3.0,PERDL)
      CALL RADOUR (CRP,FILNAM,LOGF,'RDRRTB',0.0,366.0,RDRRTB,30,IFND)
      CALL RADOUR (CRP,FILNAM,LOGF,'RDRSTB',0.0,366.0,RDRSTB,30,IFND)

C --- water use
      CALL RSDOUR (CRP,FILNAM,LOGF,'HLIM1' ,-100.0,100.0,HLIM1)
      CALL RSDOUR (CRP,FILNAM,LOGF,'HLIM2U',-1000.0,100.0,HLIM2U)
      CALL RSDOUR (CRP,FILNAM,LOGF,'HLIM2L',-1000.0,100.0,HLIM2L)
      CALL RSDOUR (CRP,FILNAM,LOGF,'HLIM3H',-10000.0,100.0,HLIM3H)
      CALL RSDOUR (CRP,FILNAM,LOGF,'HLIM3L',-10000.0,100.0,HLIM3L)
      CALL RSDOUR (CRP,FILNAM,LOGF,'HLIM4' ,-16000.0,100.0,HLIM4)
      CALL RSDOUR (CRP,FILNAM,LOGF,'RSC',0.0,1000.0,RSC)
      CALL RSDOUR (CRP,FILNAM,LOGF,'ADCRH',0.0,5.0,ADCRH)
      CALL RSDOUR (CRP,FILNAM,LOGF,'ADCRL',0.0,5.0,ADCRL)
      CALL RSDOUR (CRP,FILNAM,LOGF,'ECMAX',0.0,20.0,ECMAX)
      CALL RSDOUR (CRP,FILNAM,LOGF,'ECSLOP',0.0,40.0,ECSLOP)
      CALL RSDOUR (CRP,FILNAM,LOGF,'COFAB',0.0,1.0,COFAB)

C --- rooting
      CALL RADOUR (CRP,FILNAM,LOGF,'RDCTB',0.0,100.0,RDCTB,22,IFND)
      CALL RSDOUR (CRP,FILNAM,LOGF,'RDI',0.0,1000.0,RDI)
      CALL RSDOUR (CRP,FILNAM,LOGF,'RRI',0.0,100.0,RRI)
      CALL RSDOUR (CRP,FILNAM,LOGF,'RDC',0.0,1000.0,RDC)

C --- normalize RDCTB
      SUM = 0.0
      DO 96 X = 0.05d0,1.0d0,0.1d0
        VALUE = AFGEN (RDCTB,22,X)
        SUM = SUM+0.1*VALUE
96    CONTINUE

      RFAC = 1/SUM
      DO 98 I = 1,IFND/2
98    RDCTB(I*2) = RDCTB(I*2)*RFAC

      CLOSE (CRP)

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE RDHEA (PRJNAM,ENVSTR,LOGF,SWSHF,TAMPLI,TMEAN,DDAMP,
     &  TIMREF,TEMPI,NUMNOD)
C ----------------------------------------------------------------------
C     Date               : 29/9/99                                     
C     Purpose            : read soil heat transport input data     
C                          
C     Subroutines called : SHIFTL,SHIFTR,FINDUNIT,TTUTIL                                          
C     Functions called   : -                     
C     File usage         :                                     
C ----------------------------------------------------------------------
C---- Declarations
      IMPLICIT NONE
      Include 'Param.Fi'

      INTEGER LOGF,SWSHF,NUMNOD

      REAL*8 TAMPLI,TMEAN,DDAMP,TIMREF,TEMPI(MACP)

      CHARACTER PRJNAM*8,ENVSTR*50 ,FILNAM*12
C-----------------------------------------------------------------------
C --- local
      INTEGER HEA,CP

      REAL*8  TEMP(MACP)

      LOGICAL EXISTS
C ----------------------------------------------------------------------
C --- compose filename
      CALL SHIFTR (PRJNAM)
      FILNAM = PRJNAM//'.HEA'
      CALL SHIFTL (FILNAM)
      CALL SHIFTR (ENVSTR)

C --- check if a file with heat flow data does exist
      INQUIRE (FILE=ENVSTR//FILNAM,EXIST=EXISTS)

      IF (.NOT.EXISTS) THEN
C ---   write error to log file
        WRITE (LOGF,92) ENVSTR//FILNAM
 92     FORMAT ('SWAP can not find the input file',/,A)
        STOP 'The .HEA file was not found: SWAP stopped !'
      ENDIF

C --- open file with soil heat flow parameters
      CALL FINDUNIT(20,HEA)
      OPEN (HEA,FILE=ENVSTR//FILNAM,STATUS='OLD',
     &  ACCESS='SEQUENTIAL')

C --- method 
      CALL RSINTR (HEA,FILNAM,LOGF,'SWSHF',1,2,SWSHF)

      IF (SWSHF.EQ.1) THEN
        CALL RSDOUR (HEA,FILNAM,LOGF,'TAMPLI',0.0,50.0,TAMPLI)
        CALL RSDOUR (HEA,FILNAM,LOGF,'TMEAN',5.0,30.0,TMEAN)
        CALL RSDOUR (HEA,FILNAM,LOGF,'DDAMP',0.0,500.0,DDAMP)
        CALL RSDOUR (HEA,FILNAM,LOGF,'TIMREF',0.0,366.0,TIMREF)
	  close(hea)
	  RETURN
      ELSEIF (SWSHF.EQ.2) THEN
        CALL JMPLBL (HEA,FILNAM,LOGF,'TEMPI')
        CALL RFDOUR (HEA,FILNAM,LOGF,'TEMPI',-10.0,40.0,TEMP,MACP,
     &  NUMNOD)
        DO 30  CP = 1,NUMNOD
   30   TEMPI(CP) = TEMP(CP)
      ENDIF

      CLOSE(HEA)

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE RDKEY (LOGF,PRJNAM,ENVSTR,STASTR,BRUNY,BRUND,ERUNY,
     &  ERUND,ISEQ,FMAY,PERIOD,SWRES,OUTD,OUTY,METFIL,LAT,ALT,SWETR,
     &  SWRAI,IRGFIL,CALFIL,DRFIL,BBCFIL,OUTFIL,NOFRNS,
     &  FLCAL,SWDRA,SWSOLU,SWHEA,SWVAP,SWDRF,SWSWB,SWAFO,AFONAM,
     &  SWAUN,AUNNAM,SWATE,ATENAM,SWAIR,AIRNAM,SWSCRE,NOUTDA)
C ----------------------------------------------------------------------
C     Date               : 29/9/99       
C     Purpose            : read general input data SWAP
C     Subroutines called : SHIFTR,DATONR 
C     Functions called   : FROMTO
C     File usage         : SWAP.KEY, file with general input data
C ---------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi' 

      INTEGER LOGF,BRUND,BRUNY,ERUND,ERUNY,ISEQ,FMAY,PERIOD,SWRES
      INTEGER OUTD(1000),OUTY(1000),SWETR,SWRAI,NOFRNS,SWDRA,SWSOLU
      INTEGER SWHEA,SWVAP,SWDRF,SWSWB,SWAFO,SWAUN,SWATE,SWAIR,SWSCRE
	INTEGER NOUTDA

      REAL*8  LAT,ALT
      
      CHARACTER   PRJNAM*8,ENVSTR*50,STASTR*7,METFIL*7
      CHARACTER*8 IRGFIL(MAYRS),CALFIL(MAYRS),DRFIL(MAYRS)
      CHARACTER*8 BBCFIL(MAYRS),OUTFIL(MAYRS)
      CHARACTER*8 AFONAM,AUNNAM,ATENAM,AIRNAM

      LOGICAL FLCAL,EXISTS

C ----------------------------------------------------------------------
C --- local
      INTEGER DD(1000),MM(1000),D1,M1,Y1,IFND,SSRUN(3),ESRUN(3),KEY
      INTEGER FROMTO,I,NR,SWODAT,FDAY

      CHARACTER LINE*80 ,FILNAM*12

      LOGICAL  BLANK,COMMENT
C ----------------------------------------------------------------------

C --- check if file SWAP.KEY does exist
      INQUIRE (FILE='SWAP.KEY',EXIST=EXISTS)

      IF (.NOT.EXISTS) THEN
C ---   write error to log file
        WRITE (LOGF,92) 'SWAP.KEY'
 92     FORMAT ('SWAP can not find the input file ',A)
        STOP 'The SWAP.KEY file was not found: SWAP stopped !'
      ENDIF

C --- open key file
      CALL Findunit (10,KEY)
      FILNAM = 'SWAP.KEY'
      OPEN (KEY,FILE=FILNAM,STATUS='OLD',ACCESS='SEQUENTIAL')

C --- environment
      CALL RSCHAR (KEY,FILNAM,LOGF,'PROJECT',PRJNAM)
      CALL RSCHAR (KEY,FILNAM,LOGF,'PATH',ENVSTR)
      CALL RSINTR (KEY,FILNAM,LOGF,'CTYPE',1,2,I)
      IF (I.EQ.1) STASTR = 'UNKNOWN'
      IF (I.EQ.2) STASTR = 'NEW    '
      CALL RSINTR (KEY,FILNAM,LOGF,'SWSCRE',0,2,SWSCRE) 

C --- time variables
      CALL RFINTR (KEY,FILNAM,LOGF,'SSRUN',1,3000,SSRUN,3,3)
      D1 = SSRUN(1)
      M1 = SSRUN(2)
      Y1 = SSRUN(3)
      BRUNY = Y1
      CALL DATONR (Y1,M1,D1,BRUND)
      CALL RFINTR (KEY,FILNAM,LOGF,'ESRUN',0,3000,ESRUN,3,3)
      D1 = ESRUN(1)
      M1 = ESRUN(2)
      Y1 = ESRUN(3)
      ERUNY = Y1
      CALL DATONR (Y1,M1,D1,ERUND)
      CALL RSINTR (KEY,FILNAM,LOGF,'FMAY',1,12,FMAY)
      CALL RSINTR (KEY,FILNAM,LOGF,'PERIOD',0,366,PERIOD)
      CALL RSINTR (KEY,FILNAM,LOGF,'SWRES',0,1,SWRES)

      CALL DATONR (BRUNY,FMAY,1,FDAY) 
      ISEQ  = 0
      IF (FROMTO(FDAY,BRUNY,ERUND,ERUNY).GT.366) ISEQ=1

C --- read additional output dates
      Do 4 I = 1,1000
        OUTD(I) = 0
        OUTY(I) = 0
4     Continue

      CALL RSINTR (KEY,FILNAM,LOGF,'SWODAT',0,1,SWODAT)
      IF (SWODAT.EQ.0) THEN
        CALL JMPLBL (KEY,FILNAM,LOGF,'METFIL')
      ELSE
C ---   position file pointer
        IFND = 0
5       READ (KEY,'(A)') LINE
        CALL ANALIN (LINE,BLANK,COMMENT)
        IF (BLANK .OR. (COMMENT .AND. IFND .EQ. 0)) GOTO 5
        IF (COMMENT) GOTO 15
          IFND = IFND+1
          BACKSPACE (KEY)
          READ (KEY,*) DD(IFND),MM(IFND),OUTY(IFND) 
          CALL DATONR(OUTY(IFND),MM(IFND),DD(IFND),NR)
          OUTD(IFND) = NR
          GOTO 5
C ---   ready...
15      CONTINUE
        NOUTDA = IFND
        IF (PERIOD.EQ.0 .AND. IFND.EQ.0) Then
         PRINT *,'No output dates specified'
         STOP
        ENDIF
      ENDIF

C --- meteo
      CALL RSCHAR (KEY,FILNAM,LOGF,'METFIL',METFIL)
      CALL RSDOUR (KEY,FILNAM,LOGF,'LAT',-60.0,60.0,LAT)
      CALL RSDOUR (KEY,FILNAM,LOGF,'ALT',-400.0,3000.0,ALT)
      CALL RSINTR (KEY,FILNAM,LOGF,'SWETR',0,1,SWETR)
      CALL RSINTR (KEY,FILNAM,LOGF,'SWRAI',0,1,SWRAI) 

C --- rerun list
      IFND = 0
6     READ (KEY,'(A)') LINE
      CALL ANALIN (LINE,BLANK,COMMENT)
      IF (BLANK .OR. (COMMENT .AND. IFND .EQ. 0)) GOTO 6
      IF (COMMENT) GOTO 16
        IFND = IFND+1
        IF (IFND.GT.MAYRS) STOP 'KEY - rerun list too long'
        BACKSPACE (KEY)
        READ (KEY,*) IRGFIL(IFND),CALFIL(IFND),DRFIL(IFND),
     &                BBCFIL(IFND),OUTFIL(IFND) 
        GOTO 6
C --- ready...
16    CONTINUE
      NOFRNS = IFND
      FLCAL  = .TRUE.

C --- drainage
      CALL RSINTR (KEY,FILNAM,LOGF,'SWDRA',0,2,SWDRA)

C --- solute
      CALL RSINTR (KEY,FILNAM,LOGF,'SWSOLU',0,1,SWSOLU)

C --- heat
      CALL RSINTR (KEY,FILNAM,LOGF,'SWHEA',0,1,SWHEA)

C --- summary
      CALL RSINTR (KEY,FILNAM,LOGF,'SWVAP',0,1,SWVAP)

C --- drainage fluxes, surface reservoir 
      IF (SWDRA.EQ.2) THEN
        CALL RSINTR (KEY,FILNAM,LOGF,'SWDRF',0,1,SWDRF)
        CALL RSINTR (KEY,FILNAM,LOGF,'SWSWB',0,1,SWSWB)
      ELSE
        CALL JMPLBL (KEY,FILNAM,LOGF,'SWAFO')
      ENDIF
 
C --- output for water quality models
      CALL RSINTR (KEY,FILNAM,LOGF,'SWAFO',0,2,SWAFO)
      IF (SWAFO.GE.1) THEN
        CALL RSCHAR (KEY,FILNAM,LOGF,'AFONAM',AFONAM)
      ELSE
        CALL JMPLBL (KEY,FILNAM,LOGF,'SWAUN')
      ENDIF

      CALL RSINTR (KEY,FILNAM,LOGF,'SWAUN',0,2,SWAUN)
      IF (SWAUN.GE.1) THEN
        CALL RSCHAR (KEY,FILNAM,LOGF,'AUNNAM',AUNNAM)
      ELSE
        CALL JMPLBL (KEY,FILNAM,LOGF,'SWATE')
      ENDIF

      CALL RSINTR (KEY,FILNAM,LOGF,'SWATE',0,1,SWATE)
      IF (SWATE.EQ.1) THEN
        CALL RSCHAR (KEY,FILNAM,LOGF,'ATENAM',ATENAM)
      ELSE
        CALL JMPLBL (KEY,FILNAM,LOGF,'SWAIR')
      ENDIF
      
      CALL RSINTR (KEY,FILNAM,LOGF,'SWAIR',0,1,SWAIR)
      IF (SWAIR.EQ.1) CALL RSCHAR (KEY,FILNAM,LOGF,'AIRNAM',AIRNAM)

      CLOSE (KEY)

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE RDSOL (SOLFIL,ENVSTR,LAY,FLGENU,THETHI,THETLO,HTABLE,
     & KTABLE,THETSL,KSAT,DMCH,DMCC,COFGEN,SWSCAL,ISCLAY,FSCALE,IRUNNR,
     & KSE99,LOGF,SWHYST)
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : reading the soil physical data              
C     Subroutines called : SHIFTL,SHIFTR           
C     File usage         : SOLFIL, file containing soil characteristics        
C ----------------------------------------------------------------------
C --- global
      IMPLICIT NONE
      Include 'Param.Fi'
 
      INTEGER   LAY,SWSCAL,ISCLAY,THETLO(MAHO),THETHI(MAHO)
      INTEGER   IRUNNR,LOGF,SWHYST

      REAL*8    fscale(MAYRS)

      REAL*8    HTABLE(MAHO,99),COFGEN(8,MAHO),KTABLE(MAHO,99)
      REAL*8    DMCC(MAHO,99),DMCH(MAHO,99),THETSL(MAHO),KSAT(MAHO)
      REAL*8    KSE99(MAHO)

      CHARACTER SOLFIL*8,ENVSTR*50

      LOGICAL   FLGENU(MAHO)
C ----------------------------------------------------------------------
C --- local
      INTEGER   sol,swphys,i,j,ifnd

      REAL*8    relsat
      REAL*8    exponl,term1,THETB(99),HBUF(99),KBUF(99)

      CHARACTER filnam*12,LINE*80

      LOGICAL BLANK,COMMENT,EXISTS
C ----------------------------------------------------------------------
C --- compose filename
      CALL SHIFTR (SOLFIL)
      FILNAM = SOLFIL//'.SOL'
      CALL SHIFTL (FILNAM)
      CALL SHIFTR (ENVSTR)

C --- check if a file with soil hydraulic data does exist
      INQUIRE (FILE=ENVSTR//FILNAM,EXIST=EXISTS)

      IF (.NOT.EXISTS) THEN
C ---   write error to log file
        WRITE (LOGF,92) ENVSTR//FILNAM
 92     FORMAT ('SWAP can not find the input file: ',A)
        STOP 'The .SOL file was not found: SWAP stopped !'
      ENDIF

C --- open file with soil parameters
      CALL FINDUNIT (16,SOL)
      OPEN (SOL,FILE=ENVSTR//FILNAM,STATUS='OLD',
     & ACCESS='SEQUENTIAL')

C --- method to describe soil physical parameters
      CALL RSINTR (SOL,FILNAM,LOGF,'SWPHYS',0,1,SWPHYS)
      IF (SWPHYS.EQ.0) THEN
	  IF (SWHYST.GT.0) THEN
          WRITE(LOGF,2) FILNAM
	    WRITE(*,2) FILNAM
 2        FORMAT(' Error in file:',A,': In order to calculate ',
     &     'hysteresis, the soil hydraulic functions',/,
     &     ' should be described by the Mualem-Van Genuchten model;',/,
     &     ' please adapt the *.SOL file')
          STOP 'Check log file'
	  ENDIF
        FLGENU(LAY) = .FALSE.
      ELSE
        FLGENU(LAY) = .TRUE.
      ENDIF

C --- soil physical data presented as a table
      IF (.NOT.FLGENU(LAY)) THEN
C ---   read table
        IFND = 0
4       READ (SOL,'(A)') LINE
        CALL ANALIN (LINE,BLANK,COMMENT)
        IF (BLANK .OR. (COMMENT .AND. IFND .EQ. 0)) GOTO 4
        IF (COMMENT) GOTO 8
          IFND = IFND+1
          IF (IFND.GT.99) STOP 'RDSOL - table too long'
          BACKSPACE (SOL)
          READ (SOL,*) THETB(IFND),HBUF(IFND),KBUF(IFND)
          GOTO 4
8       CONTINUE

        THETLO(LAY) = INT(100.0D0 *THETB(1)+0.5D0)
        THETHI(LAY) = INT(100.0D0 * THETB(IFND) + 0.5D0)

C ---   saturated water content and hydraulic conductivity
        THETSL(LAY) = THETB(IFND)
        KSAT(LAY)   = KBUF(IFND)

C ---   fill HTABLE and KTABLE
        J = 1
        DO 14 I = THETLO(LAY),THETHI(LAY)
          HTABLE(LAY,I) = HBUF(J)
          KTABLE(LAY,I) = KBUF(J)
          J = J+1
14      CONTINUE

C ---   check pressure head range
        IF (HTABLE(LAY,THETLO(LAY)) .GT. -0.99D6) THEN
          WRITE(LOGF,16) FILNAM
          WRITE(*,16) FILNAM
 16       FORMAT(' Fatal error in file: ',A,
     &           ': the lowest pressure head in',/, 
     &           ' the table of the soil hydraulic functions should be',
     &           ' smaller than or equal to -1.0e6')
          STOP 'Check log file'
        ENDIF

        IF (HTABLE(LAY,THETHI(LAY)) .LT. -1.0D-6) THEN
          WRITE(LOGF,18) FILNAM
          WRITE(*,18) FILNAM
 18       FORMAT(' Fatal error in file: ',A,
     &           ': the highest pressure head in',/,
     &           ' the table of the soil hydraulic functions should be',
     &           ' equal to zero')
          STOP 'Check log file'
        ENDIF

C ---   prepare dmc-table
        DO 20 I = THETLO(LAY),THETHI(LAY)-1
          DMCH(LAY,I) = 0.5D0*(HTABLE(LAY,I)+HTABLE(LAY,I+1))
          DMCC(LAY,I) = 0.01D0/(HTABLE(LAY,I+1)-HTABLE(LAY,I))
20      CONTINUE
        DMCH(LAY,THETHI(LAY)) = 0.0D0
        DMCC(LAY,THETHI(LAY)) = DMCC(LAY,THETHI(LAY)-1)

        HTABLE(LAY,THETHI(LAY)+1) = -1.0D0*HTABLE(LAY,THETHI(LAY)-1)
        HTABLE(LAY,THETHI(LAY)+2) = -1.0D0*HTABLE(LAY,THETHI(LAY)-2) 
        KTABLE(LAY,THETHI(LAY)+1) = KTABLE(LAY,THETHI(LAY))
        KTABLE(LAY,THETHI(LAY)+2) = KTABLE(LAY,THETHI(LAY))
      ENDIF

C --- soil physical data according to Van Genuchten
      IF (FLGENU(LAY)) THEN
C ---   first position file pointer
        CALL JMPLBL(SOL,FILNAM,LOGF,'COFGEN1')

        CALL RSDOUR (SOL,FILNAM,LOGF,'COFGEN1',0.0,0.4,COFGEN(1,
     &               LAY))
        CALL RSDOUR (SOL,FILNAM,LOGF,'COFGEN2',real(COFGEN(1,LAY)),0.95,
     &               COFGEN(2,LAY))
        CALL RSDOUR (SOL,FILNAM,LOGF,'COFGEN3',1.0E-2,1.0E4,COFGEN(3,
     &  LAY))
        CALL RSDOUR (SOL,FILNAM,LOGF,'COFGEN4',0.1E-4,1.0,COFGEN(4,
     &  LAY))
        CALL RSDOUR (SOL,FILNAM,LOGF,'COFGEN5',-25.0,25.0,COFGEN(5,
     &  LAY))
        CALL RSDOUR (SOL,FILNAM,LOGF,'COFGEN6',1.0,5.0,COFGEN(6,
     &  LAY))
        IF (SWHYST .NE. 0) THEN
          CALL RSDOUR (SOL,FILNAM,LOGF,'COFGEN8',real(COFGEN(4,LAY)),
     &    1.0,COFGEN(8,LAY))
	  ENDIF

        COFGEN(7,LAY) = 1.0d0 - (1.0d0/cofgen(6,lay))
        COFGEN(5,LAY) = COFGEN(7,LAY) * (COFGEN(5,LAY)+2.0D0)

C --- modify some parameters according to scaling factors
        IF (SWSCAL.EQ.1.AND.ISCLAY.EQ.LAY) THEN
          COFGEN(3,LAY) = COFGEN(3,LAY)*FSCALE(IRUNNR)**2
          COFGEN(4,LAY) = COFGEN(4,LAY)*FSCALE(IRUNNR)
        ENDIF

C --- saturated water content and conductivity for each layer
        thetsl(lay) = cofgen(2,lay)
        ksat(lay)   = cofgen(3,lay)
        relsat      = 0.99d0
        exponl      = cofgen(5,lay)/cofgen(7,lay)-2.0d0
        term1       = (1.0d0-relsat**(1.0/cofgen(7,lay)))**cofgen(7,lay)
        kse99(lay)  = cofgen(3,lay)*(relsat**exponl)*(1.0d0-term1)*
     &                (1.0d0-term1)
      ENDIF

      CLOSE (SOL)
      RETURN
      END


C ----------------------------------------------------------------------
      SUBROUTINE RDSLT (PRJNAM,ENVSTR,LOGF,CPRE,CMLI,DDIF,LDIS,TSCF,KF,
     &  FREXP,CREF,DECPOT,GAMPAR,RTHETA,BEXP,FDEPTH,KMOBIL,
     &  HAQUIF,POROS,KFSAT,DECSAT,CDRAIN,NUMNOD,NUMLAY,SWBR)
C ----------------------------------------------------------------------
C     Date               : 29/9/99                                     
C     Purpose            : read soil water and profile characteristics     
C                          
C     Subroutines called : -                                           
C     Functions called   : -                     
C     File usage         : -                                           
C ----------------------------------------------------------------------
C---- Declarations
      IMPLICIT NONE
      Include 'Param.Fi'

      INTEGER NUMNOD,NUMLAY,LOGF,SWBR

      REAL*8 CPRE,CMLI(MACP),DDIF,LDIS,TSCF,KF,FREXP,CREF 
      REAL*8 DECPOT,GAMPAR,RTHETA,BEXP,FDEPTH(MAHO),KMOBIL,HAQUIF,POROS
      REAL*8 KFSAT,DECSAT,CDRAIN

      CHARACTER PRJNAM*8,ENVSTR*50,FILNAM*12
C ----------------------------------------------------------------------
C --- local
      INTEGER SLT,LAY,SWSP,SWDC,SWPREF

      LOGICAL EXISTS
C ----------------------------------------------------------------------
C --- compose filename
      CALL SHIFTR (PRJNAM)
      FILNAM = PRJNAM//'.SLT'
      CALL SHIFTL (FILNAM)
      CALL SHIFTR (ENVSTR)

C --- check if a file with solute transport data does exist
      INQUIRE (FILE=ENVSTR//FILNAM,EXIST=EXISTS)

      IF (.NOT.EXISTS) THEN
C ---   write error to log file
        WRITE (LOGF,92) ENVSTR//FILNAM
 92     FORMAT ('SWAP can not find the input file',/,A)
        STOP 'The .SLT file was not found: SWAP stopped !'
      ENDIF

C --- open file with solute parameters
      CALL FINDUNIT(30,SLT)
      OPEN (SLT,FILE=ENVSTR//FILNAM,STATUS='OLD',
     &  ACCESS='SEQUENTIAL')

      CALL RSDOUR (SLT,FILNAM,LOGF,'CPRE',0.0,100.0,CPRE)
      CALL RFDOUR (SLT,FILNAM,LOGF,'CMLI',0.0,1000.0,CMLI,MACP,NUMNOD)
	CALL JMPLBL (SLT,FILNAM,LOGF,'DDIF')
      CALL RSDOUR (SLT,FILNAM,LOGF,'DDIF',0.0,10.0,DDIF)
      CALL RSDOUR (SLT,FILNAM,LOGF,'LDIS',0.0,100.0,LDIS)
      CALL RSDOUR (SLT,FILNAM,LOGF,'TSCF',0.0,10.0,TSCF)

C --- sorption
      CALL RSINTR (SLT,FILNAM,LOGF,'SWSP',0,1,SWSP)
      IF (SWSP .EQ. 1) THEN
        CALL RSDOUR (SLT,FILNAM,LOGF,'KF',0.0,100.0,KF)
        CALL RSDOUR (SLT,FILNAM,LOGF,'FREXP',0.0,10.0,FREXP)
        CALL RSDOUR (SLT,FILNAM,LOGF,'CREF',0.0,1000.0,CREF)
      ELSE IF (SWSP.EQ.0) THEN
        FREXP = 1.0d0
        CREF  = 1.0d0
        KF    = 0.0
        CALL JMPLBL (SLT,FILNAM,LOGF,'SWDC')
      ENDIF

C --- decomposition
      CALL RSINTR (SLT,FILNAM,LOGF,'SWDC',0,1,SWDC)
      IF (SWDC .EQ. 1) THEN
        CALL RSDOUR (SLT,FILNAM,LOGF,'DECPOT',0.0,10.0,DECPOT)
        CALL RSDOUR (SLT,FILNAM,LOGF,'GAMPAR',0.0,0.5,GAMPAR)
        CALL RSDOUR (SLT,FILNAM,LOGF,'RTHETA',0.0,0.4,RTHETA)
        CALL RSDOUR (SLT,FILNAM,LOGF,'BEXP',0.0,2.0,BEXP)
        CALL RFDOUR (SLT,FILNAM,LOGF,'FDEPTH',0.0,1.0,FDEPTH,MAHO,
     &  NUMLAY)
      ELSE IF (SWDC .EQ. 0) THEN
        DECPOT = 0.0
        GAMPAR = 0.0
        RTHETA = 0.5d0
        BEXP   = 0.0
        DO 181 LAY = 1,MAHO
181     FDEPTH(LAY) = 0.0
        CALL JMPLBL (SLT,FILNAM,LOGF,'SWPREF')
      ENDIF

C --- preferential flow 
      CALL RSINTR (SLT,FILNAM,LOGF,'SWPREF',0,1,SWPREF)
      IF (SWPREF .EQ. 1) THEN
        CALL RSDOUR (SLT,FILNAM,LOGF,'KMOBIL',0.0,100.0,KMOBIL)
      ELSEIF (SWPREF .EQ. 0) THEN
        KMOBIL = 0.0
        CALL JMPLBL (SLT,FILNAM,LOGF,'SWBR')
      ENDIF

C --- breakthrough
      CALL RSINTR (SLT,FILNAM,LOGF,'SWBR',0,1,SWBR)
      IF (SWBR .EQ. 0) THEN
        CALL RSDOUR (SLT,FILNAM,LOGF,'CDRAIN',0.0,100.0,CDRAIN)
        HAQUIF = 100.0d0
        POROS  = 1.0d0
        KFSAT  = 0.0
        DECSAT = 0.0
      ELSEIF (SWBR .EQ. 1) THEN
        CALL JMPLBL (SLT,FILNAM,LOGF,'HAQUIF')
        CALL RSDOUR (SLT,FILNAM,LOGF,'HAQUIF',0.0,10000.0,HAQUIF)
        CALL RSDOUR (SLT,FILNAM,LOGF,'POROS',0.0,0.6,POROS)
        CALL RSDOUR (SLT,FILNAM,LOGF,'KFSAT',0.0,100.0,KFSAT)
        CALL RSDOUR (SLT,FILNAM,LOGF,'DECSAT',0.0,10.0,DECSAT)
        CALL RSDOUR (SLT,FILNAM,LOGF,'CDRAINI',0.0,100.0,CDRAIN)
      ENDIF

      CLOSE(SLT)

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE RDSWA(PRJNAM,ENVSTR,LOGF,PONDMX,SWREDU,COFRED,RSIGNI,
     &  DTMIN,DTMAX,SWNUMS,THETOL,NUMLAY,NUMNOD,BOTCOM,DZ,SOLFIL,RDS,
     &  SWHYST,TAU,SWSCAL,NSCALE,ISCLAY,FSCALE,SWMOBI,PF1,FM1,PF2,FM2,
     &  THETIM,SWCRACK,SHRINA,MOISR1,MOISRD,ZNCRACK,GEOMF,DIAMPOL,
     &  RAPCOEF,DIFDES,SWDIVD,COFANI,SWINCO,HI,GWLI,THETCR,PSAND,PSILT,
     &  PCLAY,ORGMAT,CFBS,SWCFBS)
C ----------------------------------------------------------------------
C     Date               : 29/9/99                               
C     Purpose            : read soil water and profile characteristics     
C                          
C     Subroutines called : -                                           
C     Functions called   : -                     
C     File usage         : -                                           
C ----------------------------------------------------------------------
C---- Declarations
      IMPLICIT NONE
      Include 'Param.Fi'

      INTEGER LOGF,SWREDU,SWNUMS,NUMLAY,NUMNOD,BOTCOM(MAHO),SWHYST
      INTEGER SWSCAL,NSCALE,ISCLAY,SWMOBI,SWCRACK,SWDIVD,SWINCO,SWCFBS

      REAL*8 PONDMX,COFRED,RSIGNI,DTMIN,DTMAX,THETOL,DZ(MACP),RDS
      REAL*8 FSCALE(MAYRS),PF1(MAHO),FM1(MAHO),PF2(MAHO),FM2(MAHO)
      REAL*8 MOISR1,MOISRD,ZNCRACK,GEOMF,DIAMPOL,RAPCOEF,DIFDES
      REAL*8 COFANI(MAHO),GWLI,THETCR(MAHO),PSAND(MAHO),PSILT(MAHO)
	REAL*8 PCLAY(MAHO),ORGMAT(MAHO),CFBS,SHRINA

      REAL*8 TAU,THETIM(MAHO),HI(MACP)

      CHARACTER   PRJNAM*8,ENVSTR*50
      CHARACTER*8 SOLFIL(MAHO)

C ----------------------------------------------------------------------
C --- local variables
      INTEGER SWA,IFND,LAY
      
      CHARACTER   FILNAM*12,LINE*80

      LOGICAL BLANK,COMMENT,EXISTS
C ----------------------------------------------------------------------
C --- compose filename
      CALL SHIFTR (PRJNAM)
      FILNAM = PRJNAM//'.SWA'
      CALL SHIFTL (FILNAM)
      CALL SHIFTR (ENVSTR)

C --- check if a file with soil water data does exist
      INQUIRE (FILE=ENVSTR//FILNAM,EXIST=EXISTS)

      IF (.NOT.EXISTS) THEN
C ---   write error to log file
        WRITE (LOGF,98) ENVSTR//FILNAM
 98     FORMAT ('SWAP can not find the input file',/,A)
        STOP 'The .SWA file was not found: SWAP stopped !'
      ENDIF

C --- open file with soil parameters
      CALL FINDUNIT(22,SWA)
      OPEN (SWA,FILE=ENVSTR//FILNAM,STATUS='OLD',
     & ACCESS='SEQUENTIAL')

C --- ponding
      CALL RSDOUR (SWA,FILNAM,LOGF,'PONDMX',0.0,1000.0,PONDMX)

C --- soil evaporation
      CALL RSINTR (SWA,FILNAM,LOGF,'SWCFBS',0,1,SWCFBS)
      IF (SWCFBS.EQ.1) THEN
        CALL RSDOUR (SWA,FILNAM,LOGF,'CFBS',0.5,1.5,CFBS)
      ELSE
        CALL JMPLBL(SWA,FILNAM,LOGF,'SWREDU')
      ENDIF
      CALL RSINTR (SWA,FILNAM,LOGF,'SWREDU',0,2,SWREDU)
      IF (SWREDU .GT.0) THEN
        CALL RSDOUR (SWA,FILNAM,LOGF,'COFRED',0.0,1.0,COFRED)
        CALL RSDOUR (SWA,FILNAM,LOGF,'RSIGNI',0.0,1.0,RSIGNI)  
      ELSE
        CALL JMPLBL (SWA,FILNAM,LOGF,'DTMIN')
      ENDIF
 
C --- parameters numerical scheme
      CALL RSDOUR (SWA,FILNAM,LOGF,'DTMIN',1.0E-8,0.1,DTMIN)
      CALL RSDOUR (SWA,FILNAM,LOGF,'DTMAX',0.01,0.5,DTMAX)
      CALL RSINTR (SWA,FILNAM,LOGF,'SWNUMS',1,2,SWNUMS)
      CALL RSDOUR (SWA,FILNAM,LOGF,'THETOL',1.E-5,0.01,THETOL)

C --- geometry of the soil profile
      CALL RSINTR (SWA,FILNAM,LOGF,'NUMLAY',1,MAHO,NUMLAY)
      CALL RSINTR (SWA,FILNAM,LOGF,'NUMNOD',1,MACP,NUMNOD)
      CALL RFINTR (SWA,FILNAM,LOGF,'BOTCOM',1,NUMNOD,BOTCOM,MAHO,NUMLAY)
C --- thickness of compartments
      CALL RFDOUR (SWA,FILNAM,LOGF,'DZ',1.0E-6,500.0,DZ,MACP,NUMNOD)

C --- files with soil hydraulic functions
      CALL RFCHAR (SWA,FILNAM,LOGF,'SOLFIL',SOLFIL,MAHO,NUMLAY)

C --- soil texture and organic matter per soil layer
      IFND = 0
 36   READ (SWA,'(A)') LINE
      CALL ANALIN (LINE,BLANK,COMMENT)
      IF (BLANK .OR. (COMMENT .AND. IFND .EQ. 0)) GOTO 36
      IF (COMMENT) GOTO 38
        IFND = IFND+1
        IF (IFND.GT.MAHO) STOP 'SWA - swmobi_table too long'
        BACKSPACE (SWA)
        READ (SWA,*) PSAND(IFND),PSILT(IFND),PCLAY(IFND),ORGMAT(IFND)
        GOTO 36
 38   CONTINUE

C --- rooting depth limitation
      CALL RSDOUR (SWA,FILNAM,LOGF,'RDS',1.0,1000.0,RDS)

C --- hysteresis
      CALL RSINTR (SWA,FILNAM,LOGF,'SWHYST',0,2,SWHYST)
      IF (SWHYST.GT.0) THEN
        CALL RSDOUR (SWA,FILNAM,LOGF,'TAU',0.0,1.0,TAU)
      ELSE
        CALL JMPLBL(SWA,FILNAM,LOGF,'SWSCAL')
      ENDIF

C --- scaling
      CALL RSINTR (SWA,FILNAM,LOGF,'SWSCAL',0,1,SWSCAL)
      IF (SWSCAL.EQ.1) then
	  IF (SWHYST.GT.0) THEN
          WRITE(LOGF,40) FILNAM
	    WRITE(*,40) FILNAM
 40       FORMAT(' Hysteresis in combination with scaling is not',
     &           ' allowed; adapt: ',A)
          STOP 'Check log file'
	  ENDIF
        CALL RSINTR (SWA,FILNAM,LOGF,'NSCALE',1,MAYRS,NSCALE)
        CALL RSINTR (SWA,FILNAM,LOGF,'ISCLAY',1,MAHO,ISCLAY)
        CALL RADOUR (SWA,FILNAM,LOGF,'FSCALE',0.0,100.0,FSCALE,MAYRS,
     &  IFND)
      ELSE
        CALL JMPLBL(SWA,FILNAM,LOGF,'SWMOBI')
      ENDIF

C --- preferential flow due to immobile water
      CALL RSINTR (SWA,FILNAM,LOGF,'SWMOBI',0,1,SWMOBI)
      IF (SWMOBI.EQ.1) THEN
	  IF (SWHYST.GT.0) THEN
          WRITE(LOGF,42) FILNAM
	    WRITE(*,42) FILNAM 
 42       FORMAT(' Hysteresis in combination with preferential',
     &           ' flow is not allowed; adapt: ',A)
          STOP 'Check log file'
	  ENDIF
	  IF (SWSCAL.EQ.1) THEN
          WRITE(LOGF,44) FILNAM
	    WRITE(*,44) FILNAM
 44       FORMAT(' Scaling in combination with preferential',
     &           ' flow is not allowed; adapt: ',A)
          STOP 'Check log file'
	  ENDIF
C ---   read table
        IFND = 0
4       READ (SWA,'(A)') LINE
        CALL ANALIN (LINE,BLANK,COMMENT)
        IF (BLANK .OR. (COMMENT .AND. IFND .EQ. 0)) GOTO 4
        IF (COMMENT) GOTO 8
          IFND = IFND+1
          IF (IFND.GT.MAHO) STOP 'SWA - swmobi_table too long'
          BACKSPACE (SWA)
          READ (SWA,*) PF1(IFND),FM1(IFND),PF2(IFND),FM2(IFND),
     &                  THETIM(IFND)

          GOTO 4
8       CONTINUE

      ELSE IF (SWMOBI.EQ.0) THEN
        DO 20  LAY = 1, MAHO
          PF1(LAY) = 0.0
          FM1(LAY) = 0.0
          PF2(LAY) = 0.0
          FM2(LAY) = 0.0
          THETIM(LAY) = 0.0
   20   CONTINUE
        CALL JMPLBL(SWA,FILNAM,LOGF,'SWCRACK')
      ENDIF

C --- flow through soil cracks
      CALL RSINTR (SWA,FILNAM,LOGF,'SWCRACK',0,1,SWCRACK)
      IF (SWCRACK.EQ.1) THEN
        CALL RSDOUR (SWA,FILNAM,LOGF,'SHRINA',0.0,2.0,SHRINA)
        CALL RSDOUR (SWA,FILNAM,LOGF,'MOISR1',0.0,5.0,MOISR1)
        CALL RSDOUR (SWA,FILNAM,LOGF,'MOISRD',0.0,1.0,MOISRD)
        CALL RSDOUR (SWA,FILNAM,LOGF,'ZNCRACK',-100.0,0.0,ZNCRACK)
        CALL RSDOUR (SWA,FILNAM,LOGF,'GEOMF',0.0,100.0,GEOMF)
        CALL RSDOUR (SWA,FILNAM,LOGF,'DIAMPOL',0.0,100.0,DIAMPOL)
        CALL RSDOUR (SWA,FILNAM,LOGF,'RAPCOEF',0.0,10000.0,RAPCOEF)
        CALL RSDOUR (SWA,FILNAM,LOGF,'DIFDES',0.0,10000.0,DIFDES)
        CALL RFDOUR (SWA,FILNAM,LOGF,'THETCR',0.0,1.0,THETCR,MAHO,
     &  NUMLAY)
      ELSEIF (SWCRACK.eq.0) THEN
        SHRINA = 0.0
        MOISR1 = 0.0
        MOISRD = 0.0
        ZNCRACK = 0.0
        GEOMF = 0.0
        DIAMPOL = 0.0
        RAPCOEF = 0.0
        DIFDES = 0.0
        CALL JMPLBL(SWA,FILNAM,LOGF,'SWDIVD')
      END IF

C --- fate of discharge water in a regional system
      CALL RSINTR (SWA,FILNAM,LOGF,'SWDIVD',0,1,SWDIVD)
      IF (SWDIVD.EQ.1) THEN
        CALL RFDOUR (SWA,FILNAM,LOGF,'COFANI',0.0,1000.0,COFANI,MAHO,
     &  NUMLAY)
      ELSE
        CALL JMPLBL(SWA,FILNAM,LOGF,'SWINCO')
      ENDIF

C --- initial water conditions
      CALL RSINTR (SWA,FILNAM,LOGF,'SWINCO',1,2,SWINCO)
      IF (SWINCO.EQ.1) THEN
        CALL RFDOUR (SWA,FILNAM,LOGF,'HI',-1.0E10,1.0E4,HI,MACP,NUMNOD)
      ELSEIF (SWINCO.EQ.2) THEN
        CALL JMPLBL(SWA,FILNAM,LOGF,'GWLI')
        CALL RSDOUR (SWA,FILNAM,LOGF,'GWLI',-5000.0,100.0,GWLI)
      ENDIF

      CLOSE (SWA)

      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE REDUCEVA (SWREDU,REVA,PEVA,DAYNR,DAYSTA,NRAI,
     &                     NIRD,RUNNR,ISEQ,COFRED,RSIGNI)
C ----------------------------------------------------------------------
C     Date               : 18/08/98                                         
C     Purpose            : calculates reduction of soil evaporation
C     Subroutines called : -                                           
C     Functions called   : -                                     
C     File usage         : -                                           
C ----------------------------------------------------------------------
      IMPLICIT NONE

C --- global
      INTEGER SWREDU,DAYNR,DAYSTA,RUNNR,ISEQ

      REAL*8  REVA,PEVA,NRAI,NIRD,COFRED, RSIGNI
C ----------------------------------------------------------------------
C --- local
      REAL*8  ldwet,spev,saev,saevm1

      SAVE
C ----------------------------------------------------------------------
      if (swredu.eq.0) then
C ---   no reduction ---
        reva = peva
      elseif (swredu.eq.1) then
C ---   reduction with Black model ---
        if (daynr.eq.daysta .and. (runnr.eq.1.or.iseq.eq.0)
     &      .or. (nrai+nird) .gt. RSIGNI) ldwet = 0.0
        ldwet = ldwet + 1.0d0
        reva  = COFRED * (sqrt(ldwet)-sqrt(ldwet-1.0d0))
        reva  = min(reva,peva)
      elseif (swredu.eq.2) then
C ---   reduction with Boesten and Stroosnijder model ---
        if (daynr.eq.daysta .and. (runnr.eq.1.or.iseq.eq.0)
     &      .or. (nrai+nird) .gt. RSIGNI) then
          spev = 0.0
          saev = 0.0
        endif
        if ((nrai+nird) .lt. peva) then
          spev = spev + (peva - (nrai+nird))
          saevm1 = saev
          if (spev .lt. COFRED**2) then
            saev = spev
          else
            saev = COFRED * sqrt(spev)
          endif
          reva = nrai + nird + saev - saevm1
        else
          reva = peva
          saev = max (0.0d0,saev - (nrai+nird-peva))
          if (saev .lt. (COFRED**2)) then
            spev = saev
          else
            spev = (saev/COFRED)**2
          endif
        endif
      endif

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE ROOTEX (NUMNOD,QROT,DRZ,DZ,PTRA,Z,HLIM3H,HLIM3L,HLIM4,
     & HLIM2U,H,CML,BOTCOM,HLIM2L,HLIM1,ADCRL,ADCRH,ATMDEM,
     & RDCTAB,RDM,SWSOLU,ECMAX,ECSLOP,LAYER,THETSL,THETA,QROSUM)
C ----------------------------------------------------------------------
C     Date               : 29/9/99                                          
C     Purpose            : calculates the root extraction rates (as a
C                          function of press. head) for each nodal point
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      INTEGER NUMNOD,BOTCOM(MAHO),SWSOLU,LAYER(MACP)

      REAL*8  QROT(MACP),DRZ,DZ(MACP),PTRA,Z(MACP),CML(MACP),RDCTAB(22)
      REAL*8  HLIM3H,HLIM3L,HLIM4,HLIM2U,HLIM2L,ECMAX,ECSLOP,RDM
      REAL*8  ADCRH,ADCRL,ATMDEM,HLIM1,HLIM2,HLIM3,AFGEN,QROSUM

      REAL*8  H(MACP),THETA(MACP),THETSL(MAHO)
C ----------------------------------------------------------------------
C --- local
      INTEGER node,noddrz,lay

      REAL*8  depth,SMEAN,top,bot,mid,lam,lam1,lam2,lam3,alpsol,alpwat
      REAL*8  ECsat
C ----------------------------------------------------------------------
C --- clean rtex-array
      do 10 node = 1,numnod
        qrot(node) = 0.0
10    continue
      QROSUM = 0.0

C --- skip routine if there are no roots
      if (drz.gt.-1.d-3) goto 500

C --- determine lowest compartment containing roots
      depth = 0.0
      do 20 node = 1,numnod
        noddrz = node
        depth  = depth - dz(node)
        if (drz.ge.depth-1.0d-6) goto 30
20    continue

30    SMEAN = ptra/abs(drz)

C --- calculate root extraction of the compartments
      do 100 node = 1,noddrz
        top = abs(z(node) + 0.5*dz(node))
        mid = abs(z(node))
        bot = abs(z(node) - 0.5*dz(node))
        if (z(node)- 0.5*dz(node).lt.drz) bot = abs(drz)
        if (abs(drz).lt.rdm .and. noddrz .lt. 8) then
          qrot(node) = SMEAN*(bot-top) 
        else
          lam1 = AFGEN (RDCTAB,22,top/abs(drz))
          lam2 = AFGEN (RDCTAB,22,mid/abs(drz))
          lam3 = AFGEN (RDCTAB,22,bot/abs(drz))
          lam  = (lam1+lam2+lam3)/3
          qrot(node) = SMEAN*(bot-top)*lam
        endif 
100   continue

C --- calculating critical point hlim3 according to Feddes
      hlim3=hlim3h
      if (ATMDEM.ge.ADCRL) then
        if (ATMDEM.le.ADCRH) hlim3=hlim3h+((ADCRH-ATMDEM)/(ADCRH-ADCRL))
     &  *(hlim3l-hlim3h)
      else
        hlim3 = hlim3l
      endif

C --- calculation of transpiration reduction
      hlim2 = hlim2u
      do 200 node = 1,noddrz
        alpwat = 1.0d0
        alpsol = 1.0d0

C ---   reduction due to water stress
        if (node.gt.botcom(1)) hlim2=hlim2l
C ---   waterstress/ wet circumstances
        if (h(node).le.hlim1.and.h(node).gt.hlim2) 
     &    alpwat = (hlim1-h(node))/(hlim1-hlim2)
C ---   waterstress/ dry circumstances
        if (h(node).le.hlim3.and.h(node).ge.hlim4) 
     &    alpwat = (hlim4-h(node))/(hlim4-hlim3)
C ---   no transpiration 
        if (h(node).gt.hlim1.or.h(node).lt.hlim4)
     &    alpwat = 0.0

C ---   reduction due to salt stress
        if (SWSOLU.EQ.1) then
          lay = layer(node)
          ECsat = 1.492*cml(node)*theta(node)/thetsl(lay)
          if (ECsat .lt. ECmax) then
            alpsol = 1.0d0
          else
            alpsol = (100.0d0 - (ECsat - ECmax)*ECSLOP) / 100.
            alpsol = max(0.0d0,alpsol)
          endif
        endif

        qrot(node) = qrot(node)*alpwat*alpsol
        QROSUM = qrot(node) + QROSUM
200   continue

500   return
      end

C ----------------------------------------------------------------------
      SUBROUTINE SOILTMP (NUMNOD,THETA,THETM1,THETAS,TAV,SWSHF,
     & TEMP,DZ,DISNOD,DT,TAMPLI,TMEAN,DDAMP,TIMREF,DAYNR,Z,
     & FQUARTZ,FCLAY,FORG)
C ----------------------------------------------------------------------
C     Date               : 15/10//97
C     Purpose            : calc. soil temperature profile          
C     Subroutines called : -        
C     Functions called   : -
C     File usage         : -
C ----------------------------------------------------------------------
      IMPLICIT NONE 
      INCLUDE 'PARAM.FI'

      INTEGER ErrorCode,I,NODE,NUMNOD,DAYNR,SWSHF

      REAL*8 DISNOD(MACP+1),DT,DZ(MACP),TAV,TAMPLI,TMEAN,DDAMP,TIMREF
      REAL*8 Z(MACP),FCLAY(MACP),FQUARTZ(MACP),FORG(MACP)

      REAL*8 THETA(MACP),THETM1(MACP),HEACAP(MACP),THETAS(MACP)
      REAL*8 HEACND(MACP),HEACON(MACP),TEMP(MACP),TMPOLD(MACP)
      REAL*8 QHBOT,THEAVE(MACP)
      REAL*8 thoma(MACP),thomb(MACP),thomc(MACP),thomf(MACP)

C ----------------------------------------------------------------------

C --- save old temperature profile
      do 10 node = 1,numnod
10    TMPOLD(node) = temp(node)

      if (SWSHF .eq. 2) then

C --- compute heat conductivity and capacity ---------------------------

        do 110 node = 1,numnod
110     THEAVE(node) = 0.5 * (theta(node) + thetm1(node))

C --- calculate nodal heat capacity and thermal conductivity 
        CALL DEVRIES(NUMNOD,THEAVE,THETAS,HEACAP,HEACND,FQUARTZ,
     &        FCLAY,FORG)
        HEACON(1) = HEACND(1)
        do 130 node = 2,numnod
130     HEACON(node) = 0.5 * (HEACND(node) + HEACND(node-1))

C --- Calculate new temperature profile --------------------------------

C ---   no heat flow through bottom of profile assumed
        qhbot = 0.0

C ---   calculation of coefficients for node = 1
        I = 1
C ---   temperature fixed at soil surface
        THOMA(I) = - DT * HEACON(I) / (DZ(I) * DISNOD(I))
        THOMC(I) = - DT * HEACON(I+1) / (DZ(I) * DISNOD(I+1))
        THOMB(I) = HEACAP(I) - THOMA(I) - THOMC(I)
        THOMF(I) = HEACAP(I) * TMPOLD(I) - THOMA(I) * TAV

C ---   calculation of coefficients for 2 < node < numnod
	  DO 500 I = 2,NUMNOD-1 
          THOMA(I) = - DT * HEACON(I) / (DZ(I) * DISNOD(I))
          THOMC(I) = - DT * HEACON(I+1) / (DZ(I) * DISNOD(I+1))
          THOMB(I) = HEACAP(I) - THOMA(I) - THOMC(I)
          THOMF(I) = HEACAP(I) * TMPOLD(I)
500     CONTINUE

C ---   calculation of coefficients for node = numnod
        I = NUMNOD
        THOMA(I) = - DT * HEACON(I) / (DZ(I) * DISNOD(I))
        THOMB(I) = HEACAP(I) - THOMA(I)
        THOMF(I) = HEACAP(I) * TMPOLD(I) - (QHBOT * DT)/DZ(I)

C ---   solve for vector temp a tridiagonal linear set
        Call TriDag (Numnod, thoma, thomb, thomc, thomf, Temp,ErrorCode)
        If (ErrorCode.GT.0) Stop 'Error SoilTmp/Tridag'
      else

c ---   analytical solution temperature profile ---
        do 540 i = 1,numnod
          temp(i) = tmean + tampli*(sin(0.0172*(daynr-timref+91.0d0) +
     &              z(i)/ddamp)) / exp(-z(i)/ddamp)
 540    continue

      endif
      
      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE TOTASS (IDAY,DAYL,AMAX,EFF,LAI,KDIF,AVRAD,
     $SINLD,COSLD,DTGA)
C ----------------------------------------------------------------------
C --- Author: Daniel van Kraalingen, 1986
C --- Calculates daily total gross assimilation (DTGA) by performing
C --- a Gaussian integration over time. At three different times of 
C --- the day, irradiance is computed and used to calculate the instan- 
C --- taneous canopy assimilation, whereafter integration takes place.
C --- more information on this routine is given by Spitters et al./1988
C --- Subroutines and functions called: ASSIM, RADIAT
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER I,IDAY

      REAL*8  LAI,KDIF
      REAL*8  AMAX,AVRAD,COSLD,DAYL,DTGA,EFF,FGROS,GAUSR,HOUR,PARDIF
      REAL*8  PARDIR,SINB,SINLD

      DATA    GAUSR /0.3872983d0/
C ----------------------------------------------------------------------
C --- three point gaussian integration over day
      DTGA  = 0.
      IF (AMAX.LT.1.0d-10) GOTO 20
      DO 10 I=1,3
        HOUR = 12.0d0+DAYL*0.5*(0.5d0+(I-2)*GAUSR)
C --- at a specified hour, diffuse and direct irradiance is computed
        CALL RADIAT (IDAY,HOUR,DAYL,SINLD,COSLD,AVRAD,SINB,PARDIR,
     $              PARDIF)
C --- irradiance and crop properties determine assimilation
        CALL ASSIM (AMAX,EFF,LAI,KDIF,SINB,PARDIR,PARDIF,FGROS)
        IF(I.EQ.2) FGROS=FGROS*1.6
        DTGA = DTGA+FGROS
10    CONTINUE
      DTGA  =DTGA*DAYL/3.6

20    RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE ZEROCUMU (CQROT,CQDRA,CQBOT,CQTOP,CPTRA,CPEVA,CEVAP,
     & CRUNO,CGRAI,CNRAI,CGIRD,CNIRD,CQDRAR,CQCRACK,
     & CQDRAIN,CQDRD,CSWSUPP,CSWOUT,SQBOT,SQDRA,SQSUR,SQRAP,
     & DECTOT,ROTTOT,SQPREC,SQIRRIG,CQDARCY,CSURF)
C ----------------------------------------------------------------------
C     Date               : 9/6/99                                      
C     Purpose            : set cumulative totals to 0                
C     Subroutines called : -                                           
C     Functions called   : -                                           
C     File usage         : -                                           
C ----------------------------------------------------------------------
      IMPLICIT NONE

C --- global
      REAL*8  CQROT,CQDRA,CQBOT,CQTOP,CPTRA,CPEVA,CEVAP,CRUNO
      REAL*8  CGRAI,CNRAI,CGIRD,CNIRD,CQDARCY
      REAL*8  CQDRAIN(5),CQDRD,cswsupp,cswout,CQDRAR,CQCRACK,CSURF
      REAL*8  SQBOT,SQDRA,SQSUR,SQRAP,DECTOT,ROTTOT,SQPREC,SQIRRIG

C --- local
      INTEGER i
C ----------------------------------------------------------------------
C --- setting cumulative fluxes to zero
      cqrot   = 0.0
      cqdra   = 0.0
      cqbot   = 0.0
      cqtop   = 0.0
      cqdarcy = 0.0
      cptra   = 0.0
      cpeva   = 0.0
      cevap   = 0.0
      cruno   = 0.0
      CGRAI   = 0.0
      CNRAI   = 0.0
      CGIRD   = 0.0
      CNIRD   = 0.0
      cqdrar  = 0.0
      cqcrack = 0.0

      DO 15 i = 1,5
15    cqdrain(i) = 0.0

C --- solute fluxes
      SQPREC  = 0.0
      SQIRRIG = 0.0
      SQBOT  = 0.0
      SQDRA  = 0.0
      SQSUR  = 0.0
      SQRAP  = 0.0
      DECTOT = 0.0
      ROTTOT = 0.0
      CSURF = 0.0

C --- surface water system
      cqdrd    = 0.0
      cswsupp  = 0.0
      cswout   = 0.0

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE ZEROINTR (NUMNOD,INQROT,INQ,INQDRA,IQROT,IQDRA,IINTC,
     & IPTRA,IPEVA,IEVAP,IRUNO,IPREC,IQBOT,OUTPER,IQDRAR,IQCRACK,IGRAI,
     & IGIRD,INRAI,INIRD)
C ----------------------------------------------------------------------
C     Date               : 21/9/99                                       
C     Purpose            : set intermediate totals to 0                
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'
 
C --- global
      INTEGER NUMNOD,OUTPER

      REAL*8  INQROT(MACP),INQ(MACP+1),INQDRA(5,MACP),IINTC,IPTRA,IPEVA
      REAL*8  IEVAP,IGRAI,IGIRD,INRAI,INIRD
      REAL*8  IRUNO,IPREC,IQROT,IQDRA,IQBOT,iqdrar,iqcrack
C ----------------------------------------------------------------------
C --- local
      INTEGER node,level
C ----------------------------------------------------------------------

C --- setting intermediate fluxes to zero
      DO 10 NODE = 1,NUMNOD
        INQROT(NODE) = 0.0
        INQ(NODE)    = 0.0
        DO 20 LEVEL = 1,5
          INQDRA(LEVEL,NODE) = 0.0
20      CONTINUE
10    CONTINUE
      INQ(NUMNOD+1) = 0.0
      IQROT   = 0.0
      IQDRA   = 0.0
      IINTC   = 0.0
      IPTRA   = 0.0
      IPEVA   = 0.0
      IEVAP   = 0.0
      IRUNO   = 0.0
      IPREC   = 0.0
      IGRAI   = 0.0
      IGIRD   = 0.0
      INRAI   = 0.0
      INIRD   = 0.0
      IQBOT   = 0.0
      OUTPER  = 0
      iqdrar  = 0.0
      iqcrack = 0.0

      RETURN
      END

